package seat;


import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.util.Assert;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

/**
 * An color registry maintains a mapping between symbolic color names
 * and SWT colors. 
 * <p>
 * An color registry owns all of the color objects registered
 * with it, and automatically disposes of them when the SWT Display
 * that creates the colors is disposed. Because of this, clients do not 
 * need to (indeed, must not attempt to) dispose of these colors themselves.
 * </p>
 * <p>
 * Clients may instantiate this class (it was not designed to be subclassed).
 * </p>
 * <p>
 */
public class ColorRegistry {

	private Map table = new HashMap(10);
	
	/**
	 * Creates an empty color registry.
	 * <p>
	 * There must be an SWT Display created in the current 
	 * thread before calling this method.
	 * </p>
	 */
	public ColorRegistry() {
		this(Display.getCurrent());
	}
	
	/**
	 * Creates an empty color registry.
	 */
	public ColorRegistry(Display display) {
		super();
		Assert.isNotNull(display);
		hookDisplayDispose(display);
	}
	
	/**
	 * Returns the color associated with the given key in this registry, 
	 * or <code>null</code> if none.
	 *
	 * @param key the key
	 * @return the color, or <code>null</code> if none
	 */
	public Color get(String key) {
		Color entry = (Color)table.get(key);
		if (entry == null) {
			return null;
		}
		return entry;
	}
	
	
	/**
	 * Shut downs this resource registry and disposes of all registered colors.
	 */
	private void handleDisplayDispose() {
		for (Iterator e = table.values().iterator(); e.hasNext();) {
			Color entry = (Color)e.next();
			if (entry != null) {
				entry.dispose();
			}
		}
		table = null;
	}
	
	/**
	 * Hook a dispose listener on the SWT display.
	 *
	 * @param display the Display
	 */
	private void hookDisplayDispose(Display display) {
		display.disposeExec(new Runnable() {
			public void run() {
				handleDisplayDispose();
			}
		});
	}
	
	/**
	 * Adds a color to this registry.  This method will replace the old color
	 * if there is already a color for the given key.
	 * <p>
	 * Note that a colro registry owns all of the color objects registered
	 * with it, and automatically disposes of them the SWT Display is disposed. 
	 * Because of this, clients must not register an color object
	 * that is managed by another object.
	 * </p>
	 *
	 * @param key the key
	 * @param color the color
	 */
	public void put(String key, Color color) {
		Color entry = (Color)table.get(key);
		if (entry != null) { // already exists
			entry.dispose();
		}
		table.put(key,color);
	}
	
}
