package seat;

import java.util.ArrayList;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import seat.model.algorithms.Algorithm;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.ICompressNode;
import seat.ui.preference.AlgorithmPreference;
import seat.ui.preference.DecorationPreference;
import seat.ui.preference.IconPreference;
import seat.ui.preference.TracePreference;
import seat.ui.preference.UtilityPreference;

/**
 * The main plugin class for trace exploration.
 * Algorithms are retreived from XML descriptions and cached in this class.
 */
public class SeatPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static SeatPlugin plugin;
	
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	// AlgorithmDescriptions
	private static AlgorithmDescription[] algorithmDescription;
	// Algorithm classes created from descriptions
	private static Algorithm[] algorithms; 
	// If an algorithm can process from an given subtree, it's included in compress
	private static ICompressNode[] compress;
	
	// color registry for all the colors used inside plugin
	private static ColorRegistry colorRegistry=null; 
	// Data will be set in OpenTraceWizard, used in parsing
	private SeatExtra extra=new SeatExtra(); 
	
	/**
	 * The constructor.
	 */
	public SeatPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("org.kbre.seat.SeatPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static SeatPlugin getDefault() {
		return plugin;
	}

	/**
	 * Retrieves all extensions of "Algorithm Extension Point" of platform, and 
	 * creates the classes which encapsule all the data that descript the algorithms.
	 * The AlgorithmDescriptions are cached so a second call will return quickly. 
	 * Algorithm classes will further be created from these descriptions.   
	 * @return AlgorithmDescription[] or empty if not such algorithms.
	 * @see AlgorithmDescription[]
	 */
	public static AlgorithmDescription[] getAlgorithmDescriptions() {
		if (algorithmDescription==null) {
			IPluginRegistry registry=Platform.getPluginRegistry();
			IExtensionPoint extPoint=registry.getExtensionPoint(SeatUtil.ALGORITHM_EXTENSION_POINT);
			if (extPoint==null) return new AlgorithmDescription[0];
			IConfigurationElement[] elements=extPoint.getConfigurationElements();
			algorithmDescription=new AlgorithmDescription[elements.length];
			for (int i=0;i<elements.length;i++) 
				algorithmDescription[i]= new AlgorithmDescription(elements[i]);
		}
		return algorithmDescription; 
	}
	
	/**
	 * Creates and returns Algorithms classes from AlgorithmDescriptions. 
	 * @return Algorithm[]
	 * @see Algorithm[]
	 */
	public static Algorithm[] getAlgorithms() {
		if (algorithms==null) {
			AlgorithmDescription[] ads=getAlgorithmDescriptions();
			algorithms=new Algorithm[ads.length];
			for (int i=0;i<ads.length;i++) {
				algorithms[i]=ads[i].getAlgorithm();
			}
		}
		return algorithms; 
	}
	
	/**
	 * Returns the set of Algorithms that can have the capabilty to process from a subtree 
	 * besides using the trace or the root node as input.
	 * ICompressNode[] will be used in context menu which is associated with one particular node.
	 * @return ICompressNode[]
	 * @see ICompressNode[]
	 */
	public static ICompressNode[] getCompression() {
		if (compress==null) {
			AlgorithmDescription[] ads=getAlgorithmDescriptions();
			ArrayList comp=new ArrayList();
			for (int i=0;i<ads.length;i++) {
				ICompressNode cn=ads[i].getCompressSubtree();
				if (cn!=null)
					comp.add(cn);
			}
			compress=(ICompressNode[])comp.toArray(new ICompressNode[comp.size()]);
		}
		return compress; 
	}

	/**
	 * Returns a color registry for the plugin. The first tiem use will create all the colors
	 * needed. 
	 * @return ColorRegistry
	 */
	public ColorRegistry getColorRegistry() {
		if (colorRegistry == null) {
			colorRegistry = new ColorRegistry();
			initializeColorRegistry(colorRegistry);
		}
		return colorRegistry;
	}
	
	/**
	 * Initializes colr registry and adds color from preference settings 
	 */
	protected void initializeColorRegistry(ColorRegistry reg) {
		// TODO Auto-generated method stub		
		IPreferenceStore ps=SeatPlugin.getDefault().getPreferenceStore();
		
		RGB rgb=PreferenceConverter.getColor(ps,SeatUtil.COLOR_HIGHTLIGHT);
		if (rgb==null) 
			rgb=PreferenceConverter.getDefaultColor(ps,SeatUtil.COLOR_HIGHTLIGHT);
		reg.put(SeatUtil.COLOR_HIGHTLIGHT,new Color(Display.getCurrent(),rgb));

		rgb=PreferenceConverter.getColor(ps,SeatUtil.COLOR_FILTER);
		if (rgb==null) 
			rgb=PreferenceConverter.getDefaultColor(ps,SeatUtil.COLOR_FILTER);
		reg.put(SeatUtil.COLOR_FILTER,new Color(Display.getCurrent(),rgb));
		
		rgb=PreferenceConverter.getColor(ps,SeatUtil.COLOR_PATTERN);
		if (rgb==null) 
			rgb=PreferenceConverter.getDefaultColor(ps,SeatUtil.COLOR_PATTERN);
		reg.put(SeatUtil.COLOR_PATTERN,new Color(Display.getCurrent(),rgb));
		
		rgb=PreferenceConverter.getColor(ps,SeatUtil.COLOR_UTILITY);
		if (rgb==null) 
			rgb=PreferenceConverter.getDefaultColor(ps,SeatUtil.COLOR_UTILITY);
		reg.put(SeatUtil.COLOR_UTILITY,new Color(Display.getCurrent(),rgb));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeImageRegistry(org.eclipse.jface.resource.ImageRegistry)
	 */
	protected void initializeImageRegistry(ImageRegistry reg) {
		// TODO Auto-generated method stub		
		super.initializeImageRegistry(reg);
		
		reg.put(SeatUtil.ICON_ROOT,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_ROOT));
		reg.put(SeatUtil.ICON_PACKAGE,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_PACKAGE));
		reg.put(SeatUtil.ICON_CLASS,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_CLASS));
		reg.put(SeatUtil.ICON_COMPOUND,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_COMPOUND));
		reg.put(SeatUtil.ICON_METHOD,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_METHOD));
		reg.put(SeatUtil.ICON_FILTER,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_FILTER));
		reg.put(SeatUtil.ICON_UTILITY,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_UTILITY));
		reg.put(SeatUtil.ICON_PATTERN,SeatUtil.getImageDescriptor("icons/"+SeatUtil.ICON_PATTERN));
	}
	
	/** 
	 * Sets the default data for all the preferences of this plugin. 
	 * So the plugin can always has appopriate data even the preferences
	 * are not touched by a user.
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeDefaultPreferences(org.eclipse.jface.preference.IPreferenceStore)
	 */
	protected void initializeDefaultPreferences(IPreferenceStore store) {
		// TODO Auto-generated method stub
		TracePreference.setDefaultConfig(store);
		DecorationPreference.setDefaultConfig(store);
		IconPreference.setDefaultConfig(store);
		UtilityPreference.setDefaultConfig(store);
		AlgorithmPreference.setDefaultConfig(store);
		super.initializeDefaultPreferences(store);
	}		

	/**
	 * Gets an ImageDescriptor from the image registry
	 * @param iconName: icons file name only, path is not needed.
	 * @return ImageDescriptor.
	 */
	public static ImageDescriptor getImageDescriptor(String iconName) {
		return getDefault().getImageRegistry().getDescriptor(iconName);
//		String iconPath="icons/";
//		try {
//			URL pluginUrl=getDefault().getDescriptor().getInstallURL();
//			URL url=new URL(pluginUrl,iconPath+iconName);
//			return ImageDescriptor.createFromURL(url);			
//		} catch (MalformedURLException e) {
//			return ImageDescriptor.getMissingImageDescriptor();
//		}		
	}	
	
	/**
	 * Returns the iamge cached in the iamge registry.
	 * @param name:see SeatUtil icon defintion
	 * @return
	 */	
	public static Image getImage(String name) {
		return SeatPlugin.getDefault().getImageRegistry().get(name);
	}
	
	/**
	 * Returns a color cached in the color regsitry. 
	 * @param name
	 * @return
	 */
	public static Color getColor(String name) {
		return SeatPlugin.getDefault().getColorRegistry().get(name);
	}	
	
	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= SeatPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}	
		
	/**
	 * Returns internal structure that hold temporary information data.
	 * @return 
	 */
	public SeatExtra getExtra() {
		return extra;
	}	
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugin#shutdown()
	 */
	public void shutdown() throws CoreException {
		// TODO Auto-generated method stub
		super.shutdown();
		colorRegistry=null;
	}

}
