/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: Node.java									 		     *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

 
package seat.model.core.dynamic;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import seat.model.algorithms.Criteria;
import seat.model.core.structure.BehaviouralElement;

// May 6,2004 Eric to implements ITreeNode to follow rules
// and adapt to UI needs.

public abstract class Node 
{
  private int certificate;	       		 // Used as an id (positive integer)
  private BehaviouralElement behavElem;  // Behavioural element associated with this node
  private Vector outEdges; 				 // Defines the children of this node
  private Vector inEdges; 				 // Defines the parents of this node. A node may have many parents 
  private boolean hide;				 	 // Flag that indicates of the node is hidden or not
  private PatternOccurrence pttOcc;	     // Pattern Occurrence  
  private int state;				     // State of the node 0- Ordered; 1- Unordered, 2-IgnoreRepetitions; 
  
  // algoirthimBits are added by Eric on May 2004;
  // also 4 methods: isHidden, hide, show, isHiddenBy.
  private int algorithmBits=0;            // Used to identify which algorithms are in effect
  
  /** Creates a node.  */
    
  public Node() 
  {
  	init(0);
  }

  /** Creates a node by specifying the certificate.  
   * 
   * @param certificate a unique certificate that identifies the subtree rooted at this node
   **/
    
  public Node(int certificate) 
  {
  	 init(certificate);
  }
  
  /* Inits member variables.*/
  
  private void init(int certificate)
  {
  	 this.certificate = certificate;
  	 this.outEdges = new Vector();     
  	 this.inEdges = new Vector();  
     this.hide = false;
     this.pttOcc = null;
     this.state = 0;
  }
  
  /** Returns this node's parents. This includes hidden parents
   *
   * @return an enumartion that contains the parents of this node
   */

  public Enumeration getParents()
  {
  	 Vector temp = new Vector();
  	 
 	 for (Enumeration e = inEdges.elements(); e.hasMoreElements();)
 	 {
 	 	Edge inEdge = (Edge)e.nextElement();
 	 	temp.addElement(inEdge.getFirstNode());
 	 }	
 	 return temp.elements();
  }
  
  /** Returns the parent at a given position
   *
   * @param index the position of the parent to be returned
   * @throws ArrayIndexOutOfBoundsException 
   */

  public Node getParentAt(int index)
  	throws ArrayIndexOutOfBoundsException 
  {
   	 return (Node)((Edge)inEdges.elementAt(index)).getFirstNode();
  }
  
  /** Returns the number of parents of this node
   *
   * @return number of parents of this node
   */

  public int getNumberOfParents()
  {
  	 return inEdges.size();
  }

  /** Adds a parent node to this node
   *
   * @param parent parent of this node
   *
   */
  
  public void addParent(Node parent)	
  {
	 Edge newEdge = new Edge(parent, this);
	 inEdges.addElement(newEdge);
  }

  /** Removes a parent node from this node
   *
   * @param parent parent of this node
   *
   */

  public void removeParent(Node parent)	
  {
	 boolean found = false;
	 Edge inEdge = null;
	 
	 for (Enumeration e = inEdges.elements(); e.hasMoreElements();)
 	 {
 	 	inEdge = (Edge)e.nextElement();
 	 	if (inEdge.getFirstNode() == parent)
 	 	{
 	 		found = true;
 	 		break;
 	 	}	
 	 }
 	 
 	 if (found)
 	 {
 	 	inEdge.remove();
 	 	inEdges.remove(inEdge);
 	 }	
  }

  /** Adds a child node to this node
   *
   * @param child node to be added 
   */

  public void addChild(Node child)	
  {
  	 Edge newEdge = new Edge(this, child);
  	 outEdges.addElement(newEdge);
  }
  
  /**
   * When link two nodes, say node1,node2. If using methods:
   * node1.add(node2) and node2.addParent(node1), two instances of edges which 
   * represents same edge are created. Therefore, in this situation, use:
   * addChild(Node child, Edge edge)/addParent(Node parent, Edge edge) instead 
   */
  public void addChild(Node child,Edge newEdge)	
  {
	 outEdges.addElement(newEdge);
  }
  public void addParent(Node parent,Edge newEdge)	
  {
	 inEdges.addElement(newEdge);
  }
  
  

  /** Inserts a child at a given index
   *
   * @param child node to be inserted
   * @param index position where the child node is to be inserted
   *
   * @throws ArrayIndexOutOfBoundsException if index is invalid
   */
    
  public void insertChildAt(Node child, int index)	
  	throws ArrayIndexOutOfBoundsException 
  {
  	 Edge newEdge = new Edge(this, child);
  	 outEdges.insertElementAt(newEdge, index);
  }

  /** Removes a child 
   *
   * @param child node to be removed
   */
    
  public void removeChild(Node child)	
  {
	 boolean found = false;
	 Edge outEdge = null;
	 
	 for (Enumeration e = outEdges.elements(); e.hasMoreElements();)
 	 {
 	 	outEdge = (Edge)e.nextElement();
 	 	if (outEdge.getSecondNode() == child)
 	 	{
 	 		found = true;
 	 		break;
 	 	}	
 	 }
 	 
 	 if (found)
 	 {
 	 	outEdge.remove();
 	 	outEdges.remove(outEdge);
 	 }	
  }
  
  /** Returns a child node at given a position. 
   *
   * @param index position of the node to be returned
   * @throws ArrayIndexOutOfBoundsException if index is invalid
   */
  
  // May 6,2204 Eric to return ITreeNode instead of Node
  public Node getChildAt(int index)
  	throws ArrayIndexOutOfBoundsException
  {
  	 Edge outEdge = (Edge)outEdges.elementAt(index);
   	 return ((Node)outEdge.getSecondNode());
  }

  /** Returns the number of children of this node
   *
   * @return number of children
   */
    
  public int getNumberOfChildren()
  {
  	return outEdges.size();
  }

  /** Returns all children of this node
   *
   * @return an enumeration that contains all children of this node 
   */
    
  public Enumeration getChildren()
  {
     Vector temp = new Vector();
  	 
 	 for (Enumeration e = outEdges.elements(); e.hasMoreElements();)
 	 {
 	 	Edge outEdge = (Edge)e.nextElement();
 	 	temp.addElement(outEdge.getSecondNode());
 	 }	
 	 return temp.elements();
  }

    
  /** Returns the outgoing edge between this node and the node specified as an argument
   *
   * @param node the child node
   */
  
//Commet by Eric, There may be a bug in this code. How about if there are two edges, index 1,3 
//both starting from a parent to a child(child with index 2 points to another node). 
//In this case, the edge will ALWAYS the first edge,
// The second edge will never be returned if it's desired!
// Therefore, recommend modification is to return a list of edges instead of one.
  public Edge getEdge(Node node)
  {
  	 for (Enumeration e = outEdges.elements(); e.hasMoreElements();)
 	 {
 	 	Edge outEdge = (Edge)e.nextElement();
 	 	if (outEdge.getSecondNode() == node)
 	 		return outEdge;
 	 }	
 	 return null;
  } 

// The following classes use getEdge(Edge): MethodInvocationTraceLoader
//RemoveRepetitions, and computesignature of this Node class

// added as above comments
  public Edge getEdge(int nodeIndex)
  {
  	if ((nodeIndex<0)||(nodeIndex>outEdges.size()))
  		return null;
  	else
	 return (Edge)outEdges.get(nodeIndex);
  } 

	// The default getEdge(int) can only get child edge,
	// but in some cases, we need to get a parent edge alao, so
	// I add a getparentEdge method--used in traverse Eric
  public Edge getParentEdge(int nodeIndex)
  {
	if ((nodeIndex<0)||(nodeIndex>inEdges.size()))
		return null;
	else
	 return (Edge)inEdges.get(nodeIndex);
  }
  /**
   * Exposes parent edge vector, needs to consider. used in loader to order parent edges 
   * @return
   */
  public Vector getParentEdgeVector() {
  	return inEdges;
  }
  /** Sets behavioural element
   *
   * @param behavElem behavioural element (MethodElement or Routine) 
   *
   */

  public void setBehaviouralElement(BehaviouralElement behavElem)	
  {
	this.behavElem = behavElem;
  }

  /** Returns behavioural element
   *
   * @return behavioural element (MethodElement or Routine) 
   *
   */

  public BehaviouralElement getBehaviouralElement()	
  {
	return behavElem;
  }
    
  /** Check if this node is a leaf or not
   *
   * @return True if it is a leaf, False otherwise
   *
   */

  public boolean isLeaf()
  {
  	return (outEdges.size() == 0);
  }

  /** Check if this node is the root of the trace
   *
   * @return True if it is the root, False otherwise
   *
   */

  public boolean isRoot()
  {
  	return (inEdges.size() == 0);
  }
  
  /** Sets the node certificate
   *
   * @param certificate a positive integer that represent the certificate
   */
  
  public void setCertificate(int certificate)
  {
  	this.certificate = certificate;
  }  

  /** Returns the node certificate
   *
   * @return the node certificate
   */
  
  public int getCertificate()
  {
  	return certificate;
  }  
  
  /** Computes the signature of this node.
   *
   * @return a string representing the node's signature
   */
  public String computeSignature()
  {
	Vector allCerts = new Vector();
	String buffer = "";
	
	int length=this.getNumberOfChildren();
	for (int i=0;i<length;i++) {
		Node child = (Node)getChildAt(i);
		if (!child.isHidden())
		{
		   String cert = String.valueOf(child.getCertificate());
		   if (Criteria.ignoreRepetitions == false)
		   {
			 // Count the number of repetitions
			 cert = cert + String.valueOf(((Edge)getEdge(i)).getNumberOfRepetitions());
			 allCerts.addElement(cert);
		   }
		   else
		   {
			 // Ignore the number of repetitions
			 allCerts.addElement(cert);
		   }				  
		   buffer = buffer + " " + cert;
		}				  	
	}
		
	if (Criteria.ignoreOrder && allCerts.size() > 0)  // bug in this line remove Eric:Criteria.ignoreOrder=true
	{
		// We need to sort the certificates first.
		buffer = "";
		Object[] temp = allCerts.toArray();
		Arrays.sort(temp);
		for (int i = 0; i < temp.length; i++)
			buffer = buffer + " " + (String)temp[i];
	}		
    
	buffer = this.getLabel() + " " + buffer;
	return buffer.trim();
  }  
//  public String computeSignature()
//  {
//	Vector allCerts = new Vector();
//	String buffer = "";
//
// 	for (Enumeration e = getChildren(); e.hasMoreElements();)
//    {
//       Node child = (Node)e.nextElement();
//       if (!child.isHidden())
//       {
//          String cert = String.valueOf(child.getCertificate());
//          if (Criteria.ignoreRepetitions == false)
//      	  {
//      	  	// Count the number of repetitions
//      	 	cert = cert + String.valueOf(this.getEdge(child).getNumberOfRepetitions());
//      	 	allCerts.addElement(cert);
//      	  }
//      	  else
//      	  {
//      	  	// Ignore the number of repetitions
//      	 	allCerts.addElement(cert);
//      	  }				  
//      	  buffer = buffer + " " + cert;
//       }				  
//    } 		 	
//	
//	if (Criteria.ignoreOrder = true && allCerts.size() > 0)
//    {
//    	// We need to sort the certificates first.
//    	buffer = "";
//    	Object[] temp = allCerts.toArray();
//    	Arrays.sort(temp);
//    	for (int i = 0; i < temp.length; i++)
//    		buffer = buffer + " " + (String)temp[i];
//    }		
//    
//    buffer = this.getLabel() + " " + buffer;
//    return buffer.trim();
//  }
  

  /** Hides this node. This node and the subtree rooted at it are hidden. */
    
  public void hide_old()
  {
  	 this.hide = true;
  }

  /** Shows this node. This node and the subtree rooted at it are shown. */
  
  public void show_old()
  {
  	 this.hide = false;
  }

  /** Checks if this node is hidden (logically removed)
   * 
   * @return true if this node is hidden, false otherwise
   *
   */
  public boolean isHidden_old()
  {
  	 return (hide == true);
  }
  
  
  // each algorithm will set just one bit of it
  // when an algorithm is removed(undo), substract its ID from algorithimBits 
  // When there is no algorithms in efefct, algorithimBits=0;
  public boolean isHidden()
  {
	 return (algorithmBits!= 0);  // some alg still prevent this node from being shown
  }

  // ai start from 0 till (maximum index number -1)
  public void hide(int ai)
  {
	 this.algorithmBits |= (1<<ai);
  }

  /** Shows this node. This node and the subtree rooted at it are shown. */
  
  public void show(int ai)
  {
	this.algorithmBits &= ~(1<<ai);
  }

 // Resets to 0 to show it. Clears all bits, used only in TreeTrace.reconstitute()
  public void show()
  {
	this.algorithmBits = 0;
  }
  
  public boolean isHiddenBy(int ai)
  {
			
	int b=this.algorithmBits & (1<<ai);
	return (b!=0)?true:false;
  }
  
  public int getBits() {
  	return algorithmBits;
  }
  public void setBits(int bits) {
	algorithmBits=bits;
  }
  
//  public boolean isAppliedAlgorithm(int id) {
//	return (algorithmBits&id)!=0;
//  }
//  public void addAppliedAlgorithm(int id) {
//	algorithmBits|=id;
//  }
//  public void removeAppliedAlgorithm(int id) {
//	algorithmBits-=id;
//  }
  
  /** Sets the pattern occurrence where this node is a initiator
   * 
   * @param pttOcc pattern occurrence 
   *
   */
   
  public void setPatternOccurrence(PatternOccurrence pttOcc)
  {
  	 this.pttOcc = pttOcc;
  }
  
  /** Checks if this node is an initiator of a pattern
   * 
   * @return true is this node is an initiator of a pattern, false otherwise
   *
   */
   
  public boolean isInitiator()
  {
  	 return (pttOcc != null);
  }
  
  /** Returns the node label. This method needs to be overriden by the subclasses
   *
   * @return the node label
   *
   */
    
  public abstract String getLabel(); 
  
// Marked out by Eric, mAY5 ,2004. Becasue these method are not called. 
// furthermore, process is changed from static into just public, therefore,
//the corresponding algorithm need to be created in roder to call process
// as fdone is the first method:removeSingleRepetitions()


//  /** Removews repetitions of single comprehension units. */
//  
//  public void removeSingleRepetitions()
//  {
//  	RemoveRepetitions rem=new RemoveRepetitions();
//		rem.process(this);
////  	RemoveRepetitions.process(this);
//  } 
//  
//  /** This method changes the structure of the tree rooted at this node 
//   *  by ignoring the order of calls of all its nodes.
//   */
//
//  public void ignoreOrder()
//  { 
//  	IgnoreOrder.process(this);
//  	setState(1);
//  }
//
//  /** This method restores the order.  */
//  
//  public void restoreOrder()
//  { 
//  	RestoreOrder.process(this);
//  	setState(0);
//  }
//  
//  /** This method removes nodes after a certain depth.  */
//    
//  public void limitDepth()
//  { 
//  	LimitDepth.process(this);
//  }
//
//  /** Restores the nodes.  */
//    
//  public void restoreDepth()
//  { 
//  	RestoreDepth.process(this);
//  }
//
//  /** Ignores number of repetitions.  */
//    
//  public void ignoreRep()
//  { 
//  	IgnoreRep.process(this);
//  	setState(2);
//  }
//
//  /** Restores repetitions.  */
//    
//  public void restoreRep()
//  { 
//  	RestoreRep.process(this);
//  	setState(0);
//  }
  
  /** Sets the state of the node 0- ordered 1-unordered.  */
  
  public void setState(int state)
  {
  	this.state = state;
  }
  
  /** Returns the state of the node 0- ordered 1-unordered.  */
  
  public int getState()
  {
  	return this.state;
  }
  
  /** Returns true if this node is unordered.  */
  
  public boolean isUnordered()
  {
  	return getState() == 1;
  }
  
  /** ToString. */
  
  public String toString()
  {
  	return getLabel() + " " + getCertificate();
  }
  
}

