/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: TreeTrace.java										 *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.dynamic;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import seat.io.ITraceSource;
import seat.model.algorithms.DetectPatterns;


public abstract class TreeTrace extends Trace
{
  private Node root; 			 // Contains the first call of the trace
  private Vector pttOccurrences; // Contains an occurrence of the trace pattern
  private static Hashtable cut=new Hashtable();  // Comprehension units table  
  
  /** Creates a tree based trace (e.g. routine call trace)
   *
   * @param scenario the system scenario that corresponds to this trace
   * @param file source file that defines this trace 
   */
   
  public TreeTrace(Scenario scenario, ITraceSource source) 
  {
	super(scenario, source);
	this.pttOccurrences = new Vector();
//		this.cut = new Hashtable();
  }
  
  /** Sets the root of the trace
   *
   * @param root the root of the trace
   */
    
  public void setRoot(Node root)
  {
	this.root = root;
  }
  
  /** Returns the root of the trace
   *
   * @return root of the trace
   */
   
  public Node getRoot()
  {
	return root;
  }
  
  /** Returns the comprehension unit table
   *
   * @return the comprehension unit table
   */
   
  public static Hashtable getCompUnits()
  {
	return cut;
  }
  
  
  /** Returns the number of comprehension units of the trace
   *
   * @return the number of comprehension units.
   */
   
  public static int getNumberOfCompUnits()
  {
	return cut.size();
  }
  
  /** Returns the initiator that corresponds to a given certifcate
   *
   * @param certificate the input certificate
   *
   * @return the node which represents the initiator of a given certificate,
   *		 returns null if there is no node that correspond to this certificate
   */
   
  public static Node getInitiator(int certificate)
  {
	for(Enumeration e = cut.keys(); e.hasMoreElements();)
	{
		Node node = (Node)e.nextElement();
		int cert = Integer.parseInt((String)cut.get(node));
  		
		if (cert == certificate)
			return node;
	}  
  	
	return null;
  }
  
  /** Adds a comprehension unit to the comprehension unit table (CUT)
   *
   * @param initiator the unit initiator
   * @param certificate certificate of the initiator of the comprehension unit
   */
   
  public static void addComprehensionUnit(Node initiator, int certificate)
  {
	cut.put(initiator, String.valueOf(certificate));
  }
  
  /** Returns the certificate of the given node (which is also an initiator)
   *
   * @param node the input node
   */

  public static int getCertificate(Node node)
  {
	return Integer.parseInt((String)cut.get(node));
  }
  
  /** Loads the trace into memory. This is an abstract method and needs to be overriden.
   *
   * @throws IOException in case the trace is not loaded properly
   **/
    
  public abstract void open() throws IOException;
 
  /** Adds a pattern occurrence 
   *
   * @param pttOccurrence pattern occurrence
   */
  
  public void addPatternOccurrence(PatternOccurrence pttOccurrence)
  {
	pttOccurrences.addElement(pttOccurrence);
  } 

  /** Removes a pattern occurrence 
   *
   * @param pttOccurrence pattern occurrence
   */
  
  public void removePatternOccurrence(PatternOccurrence pttOccurrence)
  {
	pttOccurrences.removeElement(pttOccurrence);
  } 
  
  public void removeAllPatternOccurrence()
	{
	  pttOccurrences.removeAllElements();
	} 
  
  /** Return all pattern occurrences of this trace
   *
   * @return enumeration of pattern occurrences
   */
  
  public Enumeration getPatternOccurrences()
  {
	return pttOccurrences.elements();
  }  

  /** Returns number of pattern occurrences
   *
   * @return number of patterns of this trace
   */
  
  public int getNumberOfPatternOccurrences()
  {
	 return pttOccurrences.size();
  }  
  
  /** detect patterns. */

  public void detectPatterns()
  {
	 DetectPatterns.process(this); 
  }	  
  
  /** Reconstitutes the original trace. */

  public void reconstitute()
  {
	 reconstituteHelper(getRoot()); 
  }	  
  
  private void reconstituteHelper(Node node)
  {
	if (node.isHidden())
		 node.show();
  					   
	for (int i = 0; i < node.getNumberOfChildren(); i++)
		reconstituteHelper((Node)node.getChildAt(i));
  }

  /** Displays the trace on the screen. For testing purposes only*/

  public void display()
  {
	 int nl = 0;
	 displayHelper(getRoot(), nl); 
  }	  
  
  private void displayHelper(Node node, int nl)
  {
  // if (node.isHidden())
  //   return;
  	
	System.out.println(node.getLabel() + " " + 
					   nl + " " + 
					   node.getCertificate() + " " +
					   node.getState() + " " +
					   node.isHidden());
  						   
	for (int i = 0; i < node.getNumberOfChildren(); i++)
	{
		Node child = (Node)node.getChildAt(i); 
  	

  						   
		displayHelper(child, nl+1);
	}	
  }
  
/*
  public void stat()
  {
	for (Enumeration e = getCUT(); e.hasMoreElements();)
	{
		String cert = (String)e.nextElement(); 
		Node node = (Node)cut.get(cert);
		System.out.println(cert + " " +
						   node.getLabel() + " "  
						   + " " + node.getNumberOfParents());
	}
  }			
*/
}

