/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: ClassElement.java										 *	
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.core.structure;

import java.util.Vector;
import java.util.Enumeration;

public class ClassElement extends Type
{
  private Vector methods;		 // Contains methods defined by this class
  private Vector superclasses;   // Contains the superclasses of this class 
  private Vector subclasses;  	 // Contains the subclasses of this class 
  private boolean interfaceFlag; // Set to True if the class is an interface
  
  /** Creates a class element object by specifying the package and class names 
   * 
   * @param pck logical package that contains this class.
   * @param name name of the class 
   *
   */
  
  public ClassElement(SoftwareSystem sys, LogicalPackage pck, String name)
  { 	
 	super(sys, pck, name);
 	methods = new Vector();
 	subclasses = new Vector();
 	superclasses = new Vector();
 	interfaceFlag = false;
 	pck.addStructuralElement(this);
  }
  
  /** Creates a class element object by specifying the class name only.
   * 
   * @param name name fo the class 
   *
   */

  public ClassElement(SoftwareSystem sys, String name)
  { 	
 	super(sys, null, name);
 	methods = new Vector();
 	subclasses = new Vector();
 	superclasses = new Vector();
 	interfaceFlag = false;
  }
  
  //---------------------------------------------------------------------
  // Subsystem that deals with the class properties
  //---------------------------------------------------------------------
  
  /** Sets the interface flag to true.
   * 
   * @param flag true if this class is an interface, false otherwise 
   *
   */

  public void setInterfaceFlag(boolean flag)
  {
  	interfaceFlag = flag;
  }
  
  /** Checks if this class is an interface.
   * 
   * @return true if this class is an interface
   *
   */

  public boolean isInterface()
  {
  	return interfaceFlag;
  }
  
  //---------------------------------------------------------------------
  // Subsystem that deals with the class methods
  //---------------------------------------------------------------------
  
  /** Adds a method to this class
   * 
   * @param method object of the class MethodElement
   *
   */

  void addMethod(MethodElement method)
  {
  	methods.addElement(method);
  }

  /** Removes a method from this class
   * 
   * @param method object of the class MethodElement
   *
   */

  void removeMethod(MethodElement method)
  {
  	methods.removeElement(method);
  	method.setClassElement(null);
  }

  /** Returns the number of methods this class contains
   * 
   * @return number of methods 
   *
   */

  public int getNumberOfMethods()
  {
  	return methods.size();
  }
  
  /** Returns all methods of this class
   * 
   * @param method object of the class MethodElement
   *
   */

  public Enumeration getMethods()
  {
  	return methods.elements();
  }
  
  /** Returns a method at the specified position 
   * 
   * @param index position of the method to be returned
   *
   * @return an objecto of the class MethodElement
   *
   * @throws ArrayIndexOutOfBoundsException if the index is negative or not less than the current size of this Vector
   */

  public MethodElement getMethodAt(int index)
  	throws ArrayIndexOutOfBoundsException 
  {
  	return (MethodElement)methods.elementAt(index);
  }
  
  /** Returns the first occurrence of the method with the specified name
   * 
   * @param name method name
   *
   * @return an object of the class MethodElement or null if the method does not exist
   */

  public MethodElement getMethod(String name)
  {
  	for(Enumeration e = getMethods(); e.hasMoreElements();)
  	{
  	  MethodElement methElem = (MethodElement)e.nextElement();
  	  if (methElem.getName().equalsIgnoreCase(name))
  		  return methElem;
  	}
  	return null;
  }
  
  //---------------------------------------------------------------------
  // Subsystem that deals with superclasses
  //---------------------------------------------------------------------
  
  /** Adds a superclass to this class
   * 
   * @param superclass superclass of this class
   *
   */

  public void addSuperclass(ClassElement superclass)
  {
  	superclasses.addElement(superclass);
  }

  /** Removes a superclass from this class
   * 
   * @param superclass superclass of this class
   *
   */

  public void removeSuperclass(ClassElement superclass)
  {
  	superclasses.removeElement(superclass);
  }

  /** Returns the number of superclasses of this class
   * 
   * @return number of superclasses
   *
   */
   
  public int getNumberOfSuperclasses()
  {
  	return superclasses.size();
  }
  
  /** Returns the superclasses of this class
   * 
   * @return an enumeration that contains the superclasses of this class
   *
   */

  public Enumeration getSuperclasses()
  {
  	return superclasses.elements();
  }

  //---------------------------------------------------------------------
  // Subsystem that deals with subclasses
  //---------------------------------------------------------------------
  
  /** Adds a subclass to this class
   * 
   * @param subclass subclass of this class
   *
   */
  
  public void addSubclass(ClassElement subclass)
  {
  	subclasses.addElement(subclass);
  }

  /** Removes a subclass from this class
   * 
   * @param subclass subclass of this class
   *
   */

  public void removeSubclass(ClassElement subclass)
  {
  	subclasses.removeElement(subclass);
  }

  /** Returns the number of subclasses of this class
   * 
   * @return number of subclasses
   *
   */
   
  public int getNumberOfSubclasses()
  {
  	return subclasses.size();
  }
  
  /** Returns the subclasses of this class
   * 
   * @return an enumeration that contains the subclasses of this class
   *
   */

  public Enumeration getSubclasses()
  {
  	return subclasses.elements();
  }
}
