/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: SoftwareSystem.java									 *
 * Package: core.structure										 *	
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.core.structure;

import java.util.Enumeration;
import java.util.Vector;

import seat.model.core.dynamic.Scenario;

public class SoftwareSystem 
{
 private String name;			// The name of the software system to explore
 private String description;	// The description of the software system 
 private String directory;		// Location of the files of the software
 private Vector modelElements;	// Structural elments that belong to this system
 private Vector scenarios;		// Scenarios of the system for dynamic analysis
 private String state;		    // state can be "parsed" or "not_parsed"
 
 private String traceType;      // Type of system traces, may contain "OO" or "ROUTINE"
 								// Then what about a system contains both oo and routine code?
 								
 /** Creates an object that represents the software system information
  * 
  * @param name name of the software system.
  * @param description description of the nature of the system.
  */
   
 public SoftwareSystem(String name, String description,String traceType)
 {
 	this.name = name;
 	this.description = description;
 	this.modelElements = new Vector();
 	this.scenarios = new Vector();
 	this.state = "not_parsed";
 	this.traceType=traceType;
 }
 
 /** Creates an object that represents the software system information without
  * specifying the description
  * 
  * @param name name of the software system.
  *
  */
  
 public SoftwareSystem(String name, String traceType)
 {
 	this(name,"",traceType);
 }
 
 public String getTraceType() {
 	return traceType;
 }
 
 /** Sets the directory where the system source code is located
  * 
  * @param directory directory name (e.g. C:\System).
  *
  */
 
 public void setDirectory(String directory)
 {
 	this.directory = directory;
 }
 
 /** Returns the directory where the system source code is located
  * 
  * @return directory name (e.g. C:\System).
  *
  */
  
 public String getDirectory()
 {
 	return directory;
 }
 
 /** Returns true is the system is parsed, false otherise
  * 
  * @return True if the system is parsed.
  */
  
 public boolean isParsed()
 {
 	return state.equals("parsed");
 }
 
 /** Indicates whether the system is parsed or not.
  * 
  * @param parse a boolean to indicate if the system is parsed or not.
  */
  
 public void parsed(boolean parse)
 {
 	if (parse)
 		this.state = "parsed";
 	else	
 		this.state = "not_parsed";
 }
 
 //------------------------------------------------------------------------
 // Subsytem that deals with model elements
 //------------------------------------------------------------------------
 
 /** Adds a model element to the software system object
  * 
  * @param modelElem model element to be added
  *
  */
  
 public void addModelElement(ModelElement modelElem)
 {
  	modelElements.addElement(modelElem);
 }

 /** Removes a model element from the software system object
  * 
  * @param modelElem model element to be added
  *
  */
  
 public void removeModelElement(ModelElement modelElem)
 {
  	modelElements.removeElement(modelElem);
 }

 /** Returns the model element that corresponds to the specified name
  * 
  * @param signature signtaure of the model element
  *
  * @return an object of the model element
  *
  */
  
 public ModelElement getModelElement(String signature)
 {
  	for(Enumeration e = getModelElements(); e.hasMoreElements();)
  	{
  		ModelElement modelElem = (ModelElement)e.nextElement();
  		if (modelElem.getName().equals(signature))
  			return modelElem;
  	}
  	return null;
 }
 
 /** Returns all model elements
  * 
  *
  * @return an enumeration of all model elements
  *
  */
  
 public Enumeration getModelElements()
 {
  	return modelElements.elements();
 }
 
 //------------------------------------------------------------------------
 // Subsytem that deals with scenarios
 //------------------------------------------------------------------------
 
 /** Adds a scenario to the software system object
  * 
  * @param scenario scenario to add
  *
  */
  
 public void addScenario(Scenario scenario)
 {
  	scenarios.addElement(scenario);
 }

 /** Removes a scenario from the software system object
  * 
  * @param scenario scenario to add
  *
  */
  
 public void removeScenario(Scenario scenario)
 {
	scenarios.removeElement(scenario);
 }

 /** Returns all scenarios of this software system
  * 
  * @return all scenarios of this system
  *
  */
  
 public Enumeration getScenarios()
 {
  	return scenarios.elements();
 }
}