/** Generic Algorithm Actions.
 *  Special algorithm will be substitute generic later on.
 */
package seat.ui.actions;

import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;

import seat.SeatUtil;
import seat.model.algorithms.Algorithm;
import seat.model.core.dynamic.Node;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.AlgorithmList;
import seat.ui.algorithm.IAlgorithmAssistant;
import seat.ui.algorithm.Parameter;
import seat.ui.tree.viewer.MultiTraceEditor;
import seat.ui.tree.viewer.TraceEditorPage;
import seat.ui.views.ModelView;

public class AlgorithmAction extends Action implements IViewUpdater{
	protected AlgorithmDescription algorithmDesc; // hold info for alg and para	
	protected Algorithm algorithm; // create and cahed for performance
		
	public AlgorithmAction(AlgorithmDescription alg) {
		super("",Action.AS_CHECK_BOX);
		algorithmDesc=alg;
		setId(alg.getID());
		setText(alg.getName());
		setToolTipText(alg.getDescription());
		setImageDescriptor(SeatUtil.getImageDescriptor(alg.getIcon()));
	}
	
	/**
	 * Main method for the action when icon is clicked or command issued.
	 * Refreshes corresponding views after the algorithm is done. 
	 * 
	 * This method includes UI related work, such as refresh views, it calls 
	 * runAlgorithm() which in turn calls another method core() to do the work.
	 */
	public void run() {
		IEditorPart ep=SeatUtil.getActiveEditor();
		Assert.isNotNull(ep);
		if (ep instanceof MultiTraceEditor) {
//			System.out.println("An algorithm runs...");
			runAlgorithm((MultiTraceEditor)ep,false);
//			System.out.println("An algorithm finished...");
			MultiActionContributor.reapply4CurrentEditor(this); // redo other effective algs

			((MultiTraceEditor)ep).refresh();
			updateView(algorithmDesc.getTargetView(),true);
		}
	}
	
	/**
	 * Applies or cancels current algorithm; or if update is true, reapply algorithm if it's already in effect
	 * @param update: false, when called by action run method, and the check/uncheck status will alternate.
	 *                true, reapply algorithm to reflect changes if this algorithm is on.
	 * This method includes calls to core algoirthm and pre/post processing class. 
	 */
	public void runAlgorithm(MultiTraceEditor ed,boolean updat) {
		final MultiTraceEditor editor=ed;
		final boolean update=updat;
		final AlgorithmAction current=this;
		BusyIndicator.showWhile(Display.getCurrent(),new Runnable() {
			public void run() {
				// pre-procesisng
				IAlgorithmAssistant pre=algorithmDesc.getPreprocess();
				if (pre!=null)
					pre.run(current);
				// core algorithm
				core(editor.getCurrentEditor(),update);
				//post processing
				IAlgorithmAssistant post=algorithmDesc.getPostprocess();
				if (post!=null)
					post.run(current);				
			}
		});
	} 
	
	/**
	 * Applies or undoes an algorithm. It is the kernal part that modifyinh the trace
	 * @param trace
	 * @param update
	 */
	public void core(TraceEditorPage edit,boolean update) {
		// prepare paras
		Parameter[] paras=algorithmDesc.getParameters();
		String index=algorithmDesc.getIndex();
		for (int i=1;i<paras.length;i++) { // o is for internal index
			paras[i].setCurrent(SeatUtil.getStringSetting(index+"@#@"+paras[i].getID()));
		}
		//load algorithm if not yet
		if (algorithm==null) 
			algorithm=algorithmDesc.getAlgorithm();		
		algorithm.init(paras);		
		AlgorithmList al=edit.getExploration().getAlgorithmList();
		Assert.isNotNull(al);
		boolean algorthimApplied=al.getAlgorithmStatus(index);
		if (update) {
			if (algorthimApplied) {
				algorithm.undo(edit.getTrace());
				algorithm.apply(edit.getTrace());
			}
			return;
		} else if (!algorthimApplied) {
			algorithm.apply(edit.getTrace());
			al.setAlgorithmStatus(index,true);			
		} else {
			algorithm.undo(edit.getTrace());
			al.setAlgorithmStatus(index,false);			
		}			
	}
	
	/**
	 * Returns the AlgorithmDescription that this action represents.
	 * @return
	 */
	public AlgorithmDescription getAlgorithmDescription() {
		return algorithmDesc;
	}

	/* (non-Javadoc)
	 * @see seat.ui.actions.IViewUpdater#updateView(java.lang.String, boolean)
	 */
	public void updateView(String viewId, boolean makeViewCurrent) {
		// TODO Auto-generated method stub
		IViewPart view;
		if (makeViewCurrent)
			view=SeatUtil.switchToView(viewId);
		else 
			view=SeatUtil.findView(viewId);
		if (view==null)
			return;

		if (view instanceof ModelView) {
			IEditorPart ep=SeatUtil.getActiveEditor();
			if (ep instanceof MultiTraceEditor) {
				TraceEditorPage page=((MultiTraceEditor)ep).getCurrentEditor();			

				ArrayList hiddenAr=new ArrayList();
				ArrayList shownAr=new ArrayList();
				ArrayList allNodes=page.getTrace().getState().getModelNodeList();					
				for (int i=0;i<allNodes.size();i++) {
					Node n=(Node)allNodes.get(i);
					if (n.isHidden())
						hiddenAr.add(n);		
					else 
						shownAr.add(n);		
				}
				ModelView model=(ModelView)view;
				model.partActivated(ep);		
				model.hideAndShowNode(hiddenAr.toArray(),shownAr.toArray());
			}				
		}				
	}

}
