
package seat.ui.algorithm;

import java.util.StringTokenizer;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import seat.SeatPlugin;
import seat.SeatUtil;
import seat.model.algorithms.Algorithm;

/**
 * This class is the representation of each algorithm contribution in the plugin.xml
 * configuration files. Actual implentation of an Algorithm class will be created 
 * through this description. 
 * @see scheme for xml descriptino of an algorithm
 */
public class AlgorithmDescription {
	private IConfigurationElement config; // a subnode of algorithm in plugin.xml
	private Algorithm algorithm;  		  // class created from config
	private ICompressNode compress;		  // if this algorithm can process a subtree 		
	private IAlgorithmAssistant preprocess,postprocess; // pre-processing and post processing class
	
	/**
	 * Constructor. Using information from xml file parsing 
	 */
	public AlgorithmDescription( IConfigurationElement cfg) {
		config=cfg; 
	}

//	public static AlgorithmDescription[] findAll(String extpntId) {
//		IPluginRegistry registry=Platform.getPluginRegistry();
//		IExtensionPoint extPoint=registry.getExtensionPoint(extpntId);
//		if (extPoint==null) return new AlgorithmDescription[0];
//		IConfigurationElement[] elements=extPoint.getConfigurationElements();
//		AlgorithmDescription[] algorithms=new AlgorithmDescription[elements.length];
//		for (int i=0;i<elements.length;i++) 
//			algorithms[i]= new AlgorithmDescription(elements[i]);
//		return algorithms; 		
//	}

	/**
	 * Find all the configured algorithms in the plugin configuration.
	 * This is actually done by delegate to SeatPlugin
	 * @return an array of AlgorithmDescription.
	 */
	public static AlgorithmDescription[] findAll() {
		return SeatPlugin.getAlgorithmDescriptions();
	}

	/**
	 * Creates an Algorithm object that this AlgorithmDescription stands for.
	 * Once the algorithm object is created, it can be used to hide/show nodes for a trace.
	 * @return- the algorithm created, or null if cannot.
	 */
	public Algorithm getAlgorithm() {
		if (algorithm!=null)
			return algorithm;
		Object alg=null;
		try {
			alg=config.createExecutableExtension("class");
		} catch (CoreException ce) {
			ce.printStackTrace();
		}
		if ((alg!=null)&&(alg instanceof Algorithm)) {
			algorithm=(Algorithm)alg;
			return algorithm; 
		} else 
			return null;
	}
	
	/**
	 * Creates an ICompressNode interface if this algorithm can process any given subtree
	 * @return ICompressNode or null if it is not.
	 * @see ICompressNode
	 */
	public ICompressNode getCompressSubtree() {
		if (compress!=null)
			return compress;
		Algorithm alg=getAlgorithm();
		if ((alg!=null)&&(alg instanceof ICompressNode)) {
			return (ICompressNode)alg;			
		} else
			return null;
	}

	/**
	 * Gets the id of algorithm.
	 * @return id
	 */
	public String getID() {
		return config.getAttribute("id");
	}

	/** 
	 * Returns the unique index(decimal number) that identifies this algorithm.
	 * This index is to be used as an id of this algorithm internally and needs to
	 * be smaller than the length of Integer.
	 * @return index as String, though it is a number
	 */
	public String getIndex() {
		return config.getAttribute("index");
	}
	
	/**
	 * Gets the name of algorithm in plugin.xml
	 * @return name
	 */
	public String getName() {
		return config.getAttribute("name");
	}
	
	/**
	 * Gets the class name that implements the algorithm.
	 * @return clas name
	 */
	public String getClassName() {
		return config.getAttribute("class");
	}
	
	/**
	 * Returns the alogirthm category: OO, Routine or both
	 * @return String category
	 */
	public String getCategory() {
		return config.getAttribute("category");		
	}

	/**
	 * Returns the icon name in default "icons" sub directory
	 * @return: the icon name
	 */
	public String getIcon() {
		return config.getAttribute("icon");		
	}
	
	/**
	 * Returns the description of algorithm in plugin.xml. The description 
	 * can span multiple line and '\t's are removed. 
	 * Note to programemr : don't use split() method as lower version of Java
	 * does not support it.  
	 * @return description
	 */
	public String getDescription() {
		IConfigurationElement[] children=config.getChildren("description");
		String description="";
		if (children!=null) {
			for (int i=0;i<children.length;i++)
				description+=children[i].getValue();
		}
		StringTokenizer st=new StringTokenizer(description,"\t");
		StringBuffer tip=new StringBuffer("");
		while (st.hasMoreTokens()) 
			tip.append(st.nextToken());
		return tip.toString();
	}
	
	/**
	 * Returns an array of Parameters of algorithm in plugin.xml
	 * Parameter objects are used to setup prefernce pages in UI.
	 * The data contained in eahc parameter is the default data parsed
	 * xml description and will be set and load from data from trace prefernce.
	 * @return Parameter[].
	 * @see Parameter
	 */
	public Parameter[] getParameters() {
		IConfigurationElement[] children=config.getChildren("parameter");
		Parameter[] paras=new Parameter[0]; // null
		if (children!=null) {
			paras=new Parameter[children.length+1]; // reserved 1 for index
			paras[0]=new Parameter("index","index","internal",getIndex());
			for (int i=0;i<children.length;i++) {
				String name=children[i].getAttribute("name");
				String id=children[i].getAttribute("id");
				String type=children[i].getAttribute("type");
				String def=children[i].getAttribute("default");
				if ((name==null)||(id==null)||(type==""))
					paras[i+1]=new Parameter("","","","");
				else {
					if (def==null)
						def="";
					paras[i+1]=new Parameter(name,id,type,def);
				}
			}
		}
		return paras;
	}

	public IAlgorithmAssistant getPreprocess() {
		if (preprocess!=null) // test if already done
			return preprocess;
		if (config.getAttribute("preprocess")==null) // exist?
			return null;	
			
		Object extra=null;
		try {
			extra=config.createExecutableExtension("preprocess");
		} catch (CoreException ce) {
			return null; 
		}
		if ((extra!=null)&&(extra instanceof IAlgorithmAssistant)) {
			preprocess=(IAlgorithmAssistant)extra;
			return preprocess; 
		} else 
			return null;
	}
	
	public IAlgorithmAssistant getPostprocess() {
		if (postprocess!=null) // already done post
			return postprocess;
		if (config.getAttribute("postprocess")==null) // exist?
			return null;	

		Object extra=null;
		try {
			extra=config.createExecutableExtension("postprocess");
		} catch (CoreException ce) {
			return null;
		}
		if ((extra!=null)&&(extra instanceof IAlgorithmAssistant)) {
			postprocess=(IAlgorithmAssistant)extra;
			return postprocess; 
		} else 
			return null;		
	}
	
	public String getTargetView() {
		String id=config.getAttribute("targetView");
		if (id==null)
			id=SeatUtil.VIEW_MODEL;
		return id;		
	}
					
}
