package seat.ui.algorithm;

import java.util.ArrayList;

import org.eclipse.core.resources.IProject;
import org.eclipse.swt.widgets.Display;

import seat.model.core.dynamic.Trace;

/**
 * This class is the abstract base class for a source parsing, such as parsing
 * Java/C++ code. It defines methods that any concrete parser must be implemented.
 * The parse()method will call these framework methods in a separate thread and
 * notify listeners of parsing finished event.
 * 
 * Algorithms that need parsing include: ConstructorDestrcutor, Polymorphism
 * and ProvateMethodRemoval for the time being. 
 */
public abstract class Parser {
	protected Trace trace;
	protected IProject[] sourceProjects;
	protected IProject activeProject;
	protected ArrayList parseListeners;
	
	/**
	 * Set trace to be parsed and the projects assocaite with the trace.
	 * @param trace- trace 
	 * @param active- active(main) project for the trace, it will first be looked when parsing
	 * @param sourcePrj- a list of additional projects, can include the active project.
	 */
	public Parser(Trace trace,IProject active, IProject[] sourcePrj) {
		// TODO Auto-generated constructor stub
		this.trace=trace;
		this.activeProject=active;
		this.sourceProjects=sourcePrj;
		if (this.sourceProjects==null)
			this.sourceProjects=new IProject[0];
	}

	/**
	 * Parses and constructs necessary relations in a separate thread.
	 * Extending this methed will not get extra processign to run in 
	 * the same thread. For that pupose, a hook method called
	 * hookMoreParsing() which can be overided if more action need to
	 * be done in parser thread.
	 *
	 */
	public void parse() {
		// As the listeners may have UI related work, called in another 
		// thread pther than parser, for quick response.		
		final Thread notifierThread = new Thread(new Runnable() {
			public void run() {
				notifyListener();		
				System.out.println("Notify thread ends.");						
			}
		});
		
		// Seperate parsing thread for caller UI code, such as createPartControl for a view
		Thread offlineThread = new Thread(new Runnable() {
			public void run() {
				// TODO Auto-generated method stub
				long startTime=0,endTime=0;
				startTime = System.currentTimeMillis();
				setPackageClassRelation();
				setClassHierarchy();
				setClassIsInterface();
				setMethodAttribute();	   	
				endTime = System.currentTimeMillis();
				trace.getState().setLoadTime(trace.getState().getLoadTime()+(float)((endTime - startTime)/1000.0));
				notifierThread.start();
				hookMoreParsing();
				System.out.println("Parser thread ends (time: "+ (float)((endTime - startTime)/1000.0));						
			}
		});
		offlineThread.start();		
	}

	/**
	 * A hook method called in the parser thread. By default, it does nothing. 
	 * It can be extended if more parsing actions needs to done in the parser thread
	 *
	 */
	public void hookMoreParsing() {
	}
	/**
	 * Adds listners for a SourceParseEvent.
	 * @param listener
	 */
	public void addParseListener(ParseListener listener) {
		if (parseListeners==null)
			parseListeners=new ArrayList(); 		
		if (!parseListeners.contains(listener)&&(listener!=null))
			parseListeners.add(listener);
	}

	/**
	 * Notifies any listners when parsing is done. Some listeners can be UI elements
	 * so an asyn call of notify is needed.
	 */	
	private void notifyListener() {
		Display.getDefault ().asyncExec (new Runnable () {
			public void run () {
				if (parseListeners!=null) {
					for (int i=0;i<parseListeners.size();i++)
					((ParseListener)parseListeners.get(i)).sourceParsed(new SourceParseEvent(trace));
				}
			}
		 });
	}

	
	/**
	 * Sets the relations between packages, and between clas and package
	 */
	public abstract void setPackageClassRelation();
	/**
	 * Set class hierarchy, superclasses and subclasses. Used in 
	 * overload and override algorithms(Polymorphism)
	 */
	public abstract void setClassHierarchy();
	
	/**
	 * Tests and sets if a class is only for interface purpose.
	 * This methods should interate all the classes currently used 
	 * and set related ClassElement attributes 
	 *
	 */
	public abstract void setClassIsInterface();

	/**
	 * Sets all methods attributes, mainly booleans inside MethodElemets:
	 * PRIVATE, PUBLIC, STATIC,etc. Algorithms that use these attibutes include
	 * ConsDes,PrivateRemovel.... 
	 *
	 */
	public abstract void setMethodAttribute();	
	
}