package seat.ui.preference;

import java.util.ArrayList;
import java.util.StringTokenizer;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.ui.algorithm.AlgorithmDescription;

/**
 * A field editor for displaying and storing a list of strings. 
 * Buttons are provided for adding items to the list and removing 
 * items from the list.
 */
public class TableFieldEditor extends FieldEditor {
	public static final int ICONS= 4;
	public static String[] columnNames = new String[] {"Algorithm","TL","TR","BL","BR" };

	private static final String DEFAULT_SEPERATOR = "$";

	private static final int VERTICAL_DIALOG_UNITS_PER_CHAR = 8;
	private static final int HORIZONTAL_DIALOG_UNITS_PER_CHAR = 4;
	private static final int LIST_HEIGHT_IN_CHARS = 20;
	private static final int LIST_HEIGHT_IN_DLUS = 
		LIST_HEIGHT_IN_CHARS * VERTICAL_DIALOG_UNITS_PER_CHAR;

	// The top-level control for the field editor.
	private Composite top;
	// The list of tags.
	private Table table;
	private TableViewer tableViewer;
	// in a single String representation.
	private String seperator = DEFAULT_SEPERATOR;
	private String defaultConfig;
	private static IconConfig[] data;	
	class IconConfig {
		String name;
		String icon;
		Boolean icon1,icon2,icon3,icon4;
		IconConfig(String n,String i) {
			name=n;
			icon=i;
			icon1=new Boolean(false);
			icon2=new Boolean(false);
			icon3=new Boolean(false);
			icon4=new Boolean(false);
		}
		IconConfig(String n,String i,Boolean i1,Boolean i2,Boolean i3, Boolean i4) {
			name=n;
			icon=i;
			icon1=i1;
			icon2=i2;
			icon3=i3;
			icon4=i4;
		}
	}
	
	public TableFieldEditor(
		String name,
		String labelText,
		Composite parent) {
			super(name, labelText, parent);		
			AlgorithmDescription[] algs=SeatPlugin.getAlgorithmDescriptions();
			data=new IconConfig[algs.length];
			for (int i=0;i<algs.length;i++)
				data[i]=new IconConfig(algs[i].getName(),algs[i].getIcon());
			defaultConfig=createTableString();
	}
	
	/**
	 * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
	 */
	protected void adjustForNumColumns(int numColumns) {
		((GridData)top.getLayoutData()).horizontalSpan = numColumns;
	}

	/**
	 * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid
	 * (Composite, int)
	 */
	protected void doFillIntoGrid(Composite parent, int numColumns) {
		top = parent;
	
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = numColumns;
		top.setLayoutData(gd);
		
		createTable(top);
		
		// Create a grid data that takes up the extra 
		// space in the dialog and spans both columns.
		GridData listData = new GridData(GridData.FILL_HORIZONTAL);
		listData.heightHint = 
			convertVerticalDLUsToPixels(table, LIST_HEIGHT_IN_DLUS);
		listData.horizontalSpan = numColumns;
		
		table.setLayoutData(listData);
//		table.addSelectionListener(new SelectionAdapter() {
//			public void widgetSelected(SelectionEvent e) {
//				selectionChanged();
//			}
//		});
		
	}
	
	private void createTable(Composite parent) {
		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
					SWT.FULL_SELECTION | SWT.HIDE_SELECTION;	
		table = new Table(top, style);
		table.setLinesVisible (true);
		table.setHeaderVisible (true);
		
		TableColumn column = new TableColumn (table, SWT.LEFT,0);
		column.setText (columnNames[0]);
		column.setWidth (200);
		for (int i=0;i<ICONS;i++) {
			column = new TableColumn (table, SWT.LEFT,i+1);
			column.setText(columnNames[i+1]);
			column.setImage(getOverlayedImage("icons/class_obj.gif","icons/ovl"+i+".gif",i));
			column.setWidth (60);
		}
		
		tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);	
		tableViewer.setColumnProperties(columnNames);

		// Create the cell editors
		CellEditor[] editors = new CellEditor[ICONS+1];

		TextCellEditor textEditor = new TextCellEditor(table);
//		textEditor.
		editors[0] = textEditor;

		for (int k=1;k<=ICONS;k++) 
			editors[k] = new CheckboxCellEditor(table);
			
		tableViewer.setCellEditors(editors);		
		tableViewer.setCellModifier(new TableCellModifier(tableViewer,columnNames));
		tableViewer.setContentProvider(new TableContentProvider());
		tableViewer.setLabelProvider(new TableLabelProvider());
		tableViewer.setInput(data);
		tableViewer.refresh();
	}
	
	private Image getOverlayedImage(String base, String overlay, int pos) {
		Image baseImage = SeatUtil.getImageDescriptor(base).createImage();
		ArrayList decors=new ArrayList();
		for (int i=0;i<pos;i++) 
			decors.add(null); 
		decors.add(SeatUtil.getImageDescriptor(overlay));
		DecoratorOverlayIcon overlayIcon = new DecoratorOverlayIcon(baseImage, decors);
		return overlayIcon.createImage();
	}
	
	public void refreshMe() {
		tableViewer.setInput(data);
		tableViewer.refresh();		
	}

	class TableContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			return data;
		}
	}

	class TableLabelProvider extends LabelProvider implements ITableLabelProvider {

		/** 
		 * Returns the source name of Exploration as labels
		 * See Exploration 
		 */
		public String getColumnText(Object obj, int index) {
			if (obj instanceof IconConfig) { 
				switch (index) {
					case 0: 
						return ((IconConfig)obj).name;
					default:
						return "";
				}
			} else
				return obj.toString();
		}

		/** 
		 * Returns a image for a given column for the exploration view. The primary viewer in
		 * exploration view is a table viewer.   
		 */
		public Image getColumnImage(Object obj, int index) {
			IconConfig ic=(IconConfig)obj;
			switch (index) {
				case 0:
					return SeatUtil.getImageDescriptor(ic.icon).createImage();
				case 1:
					return getCheckImage(ic.icon1);
				case 2:
					return getCheckImage(ic.icon2);
				case 3:
					return getCheckImage(ic.icon3);
				case 4:
					return getCheckImage(ic.icon4);
				default:
					return null;
			}
		}
		
		private Image getCheckImage(Boolean isSelected) {
			String key = isSelected.booleanValue() ? "icons/checked.gif": "icons/unchecked.gif";
			return  SeatUtil.getImageDescriptor(key).createImage();
		}
	}



	/**
	 * @see org.eclipse.jface.preference.FieldEditor#doLoad()
	 */
	protected void doLoad() {
		String items = getPreferenceStore().getString(getPreferenceName());
		setTable(items);
	}
	
	/**
	 * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
	 */
	protected void doLoadDefault() {
		setTable(defaultConfig);
	}
	
	// Parses the string into seperate list items and adds them to the list.
	private void setTable(String items) {
		IconConfig[] saved= parseString(items);
		if (saved.length==data.length) {
			// see any name changes
			int i=0;
			for (i=0;i<saved.length;i++)
				if (!saved[i].name.equals(data[i].name)) 
					break;
			if (i==saved.length)
				data=saved;							
		}
		tableViewer.refresh();
	}
	
	/**
	 * @see org.eclipse.jface.preference.FieldEditor#doStore()
	 */
	protected void doStore() {
		String s = createTableString();
		if (s != null)
			getPreferenceStore().setValue(getPreferenceName(), s);
	}

	/**
	 * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
	 */
	public int getNumberOfControls() {
		// The button composite and the text field.
		return 1;
	}

	/**
	 *  Creates the single String representation of the list
	 * that is stored in the preference store.
	 */
	private String createTableString() {
		StringBuffer path = new StringBuffer("");
		
		for (int i = 0; i < data.length; i++) {
			path.append(data[i].name);
			path.append(seperator);
			path.append(data[i].icon);
			path.append(seperator);
			path.append(data[i].icon1.toString());
			path.append(seperator);
			path.append(data[i].icon2.toString());
			path.append(seperator);
			path.append(data[i].icon3.toString());
			path.append(seperator);
			path.append(data[i].icon4.toString());
			path.append(seperator);
		}
		return path.toString();
	}
	
	/**
	 *  Parses the single String representation of the list
	 * into an array of list items.
	 */
	private IconConfig[] parseString(String stringList) {
		StringTokenizer st = 
			new StringTokenizer(stringList, seperator); //$NON-NLS-1$
		ArrayList v = new ArrayList();
		String name="", icon="";
		Boolean i1,i2,i3,i4;
		i1=i2=i3=i4=new Boolean(false);
		while (st.hasMoreElements()) {
			name=(String)st.nextElement();
			if (st.hasMoreElements()) icon=(String)st.nextElement();
			if (st.hasMoreElements()) i1=new Boolean((String)st.nextElement());
			if (st.hasMoreElements()) i2=new Boolean((String)st.nextElement());
			if (st.hasMoreElements()) i3=new Boolean((String)st.nextElement());
			if (st.hasMoreElements()) i4=new Boolean((String)st.nextElement());
			v.add(new IconConfig(name,icon,i1,i2,i3,i4));
		}
		return (IconConfig[])v.toArray(new IconConfig[v.size()]);
	}
	
}
