package seat.ui.tree.partial;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import seat.model.core.dynamic.Edge;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.tree.widget.PictureLabel;

/**
 * Core class to represent a partial tree, which is generated as the user navigate the whole tree.
 * This subtree only contains nodes that are needs for current foucs and of size equals sizeNeeded.
 * Also the links nodes that are parents of the above nodes are included. 
 * 
 * This implementation searches current children, siblings and parents and links necessary nodes
 * to contruct the partial tree. 
 */

public class PartialTree implements IDynamicTree {
	private UniquePath pathForward;     // first path after skip n path forward
	private UniquePath pathBackward;    // first path after skip n path backward

	private Hashtable expandedSet;		// contained nodes that are expanded and their state
	private Trace trace;				// trace ctf graph
	private int sizeNeeded;				// size of number of nodes needed to be retreived
	private boolean keepLastTree;		// internal flag to prevent further loading
	private boolean keepPrevTree;		// internal flag to prevent further loading
	private boolean includeHidden;		// flag to consider hidden nodes when processing
	private boolean filter;				// user specidied filter
	
	private UniquePath top4NextRound;	// will be top tree item in the UI window prepared for next display
	private int lastSkippedCounter;	  	// how many nodes are skipped in last move	

	protected UniquePath starting;		 		// this is the main()- starting point of partial tree 
	protected UniquePath invisibleRoot; 			// this will be the root of UI tree to be returned
	public Edge virtualEdge;			// edge to main method
	/** 
	 * Given graph root, initialize data.
	 * @param root
	 */
	public PartialTree(Trace trace) {
		pathForward=pathBackward=null;
		expandedSet=new Hashtable();
		this.trace=trace; 
		sizeNeeded=0;
		keepLastTree=false;
		lastSkippedCounter=0;
		includeHidden=true;
		filter=true;
		
		Edge toMain=new Edge(null,((TreeTrace)trace).getRoot());
		toMain.setStartingLineNumber(1);
		starting=new UniquePath(toMain);
		starting.setDescription("starting");
		starting.setTrace(trace);
		this.addToExpandSet(starting); // always open and add main method
		virtualEdge=toMain;

		Edge temp=new Edge(null,((TreeTrace)trace).getRoot());			
		temp.setStartingLineNumber(0);
		invisibleRoot=new UniquePath(temp); // root of UI tree to be returned, it's the same path as starting
		invisibleRoot.setDescription("Invisible");		
		top4NextRound=null;
		
		// set static var for all path-- modify to be private
		invisibleRoot.setTrace(trace);
//		lastColoredModelNode=new Hashtable();
	}

	/**
	 * Returns the tree as a list so they can be easily rendered.
	 * @return
	 */
	public List asList() {
			return asList(starting);
	}
	public static List asList(TreeNode root) {
		List list=new ArrayList();
		list.add(root);
		TreeNode[] children=root.getChildren();		
		for (int i=0;i<children.length;i++) {
			list.addAll(asList(children[i]));
		}
		return list;			
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getNextTree(partialtree.views.UniquePath, int)
	 */
	public UniquePath getNextTree(UniquePath currentNode, int offset,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingForward(currentNode,sizeNeeded,Math.abs(offset),force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getNextTree(partialtree.views.UniquePath)
	 */
	public UniquePath getNextTree(UniquePath currentNode,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingForward(currentNode,sizeNeeded,1,force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getPreviousTree(partialtree.views.UniquePath, int)
	 */
	public UniquePath getPreviousTree(UniquePath currentNode, int offset,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingBackward(currentNode,sizeNeeded,Math.abs(offset),force);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#getPreviousTree(partialtree.views.UniquePath)
	 */
	public UniquePath getPreviousTree(UniquePath currentNode,boolean force) {
		// TODO Auto-generated method stub
		return this.loadingBackward(currentNode,sizeNeeded,1,force);
	}

	/**
	 * Gets tree with nodes only between two given nodes.
	 * @param from: starting
	 * @param till: end
	 * @return:     root
	 */
	public UniquePath getTreeBetween(UniquePath from, UniquePath till) {
		// TODO Auto-generated method stub
		if (from.getLineNo()<till.getLineNo()) 
			return loadingForward(from,till);
		else
			return loadingForward(till,from);
	}

	/* (non-Javadoc)
	 * @see partialtree.views.IDynamicTree#setTreeSize(int)
	 */
	public void setTreeSize(int sizeToBeRetrieved) {
		// TODO Auto-generated method stub
		sizeNeeded=sizeToBeRetrieved;
	}

	/**
	 * Sets if hidden nodes are included when internally retrieving the nodes. 
	 * @param flag
	 */
	public void setIncludeHidden(boolean flag) {
		// TODO Auto-generated method stub
		includeHidden=flag;
	}

	
	/**
	 * Using current path and epxanded set to skip to new path.
	 * Current is included in the skipped counter.
	 * The resultant path is save in pathForward. Current node 
	 * is considered as a subtree. Nodes that are opened(expanded) before
	 * are traversed recursively. Hidden nodes are not counted.
	 * @param current: staring root of the subtree
	 * @param wanted:  Number of nodes to skip 	
	 * @return; Actual number of nodes skipped.
	 */
	private int skipNodeForward(UniquePath current, int wanted) {
		int skippedCounter=0; 
		if (isIncluded(current)) {
			skippedCounter=1;
			pathForward=current;
			if ( (skippedCounter>=wanted) || (isExpanded(current)==-1) ) {
				return skippedCounter;
			}
		} else 
			return skippedCounter;
		
		Node focused=current.getElement(),child=null;
		int length=focused.getNumberOfChildren();
		for (int i=0;i<length;i++) {
//			child=focused.getChildAt(i);
//			if (!includeHidden&&child.isHidden())  // hidden child is not counted 
//				continue;
				
			UniquePath childPath=current.createChildPath(focused.getEdge(i));
			if (!isIncluded(childPath))
				continue;
			if (isExpanded(childPath)==-1) {
				skippedCounter++;
				pathForward=childPath;
				if (skippedCounter>=wanted) {
					return skippedCounter;				
				}
			} else { // recursive
				int childSkip=skipNodeForward(childPath,wanted-skippedCounter);
				skippedCounter+=childSkip;
				if (skippedCounter>=wanted) 
					return skippedCounter;
			}				
		} // loop for
	return skippedCounter;
	}
	
	/**
	 * Traverses the siblings that are after current path, retrieve wanted path.
	 * Calls processNodeForward(), so actually pointer pathForward is moved inside processNodeForward()
	 * @param current: current Path 
	 * @param wanted: number of nodes to be skipped
	 * @return: actual number found
	 */
	private int skipSiblingsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getNextSibling();
		while (sibling!=null) {
			int sibSkip=skipNodeForward(sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getNextSibling();			
		}
		return skippedCounter;
	}
	
	/** 
	 * Loops through all parents if there is not enough nodes.
	 * Calls processSiblingsForward()
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int skipParentsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath upper1Path=current.createParentPath();
		while (upper1Path!=null) { 
			skippedCounter+=skipSiblingsForward(upper1Path,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			upper1Path=upper1Path.createParentPath();
		} // if not enough, recursively search parents			
		return skippedCounter;
	}
				
	/**
	 * Main skip method to skip given number of nodes.
	 * Delegates calls to skipNode, skipSibling and skipParent.
	 * After skipping, the next path for loading/linking is saved in pathForward.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number skipped
	 */
	private int skipForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		if (!isIncluded(current))
			return skippedCounter;
		int toSkip=wanted+1; // +1 for the next availible position
		// skip current node first
		skippedCounter+=skipNodeForward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// skip all siblings
			skippedCounter+=skipSiblingsForward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// if still not, skip all the parents
			skippedCounter+=skipParentsForward(current,toSkip-skippedCounter);
		
		if (skippedCounter>wanted)
			skippedCounter=wanted;
		return skippedCounter;
	}
	
	/**
	 * Tests if there are enough nodes can be skipped.
	 * Delegates call to skipForward by saving/restoring pathForward
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number that can be skipped
	 */
	private int testSkipForward(UniquePath current, int wanted) {
		UniquePath old=pathForward;
		int skipped=skipForward(current, wanted);
		pathForward=old;
		return skipped;
	}
	/**
	 * Tests if there is no more nodes. No pointer will be moved.
	 * @return: true- bottom
	 */
//	private boolean testBottom() {
//		UniquePath old=pathForward;
//		int skipped=skipForward(pathForward, 1);
//		pathForward=old;		
//		return (skipped==0)?true:false;
//	}
	
	/**
	 * Tries to skip certain number of nodes starting from current node.
	 * Nodes are traversed from lowest level(bottom) backwardly. Hidden nodes
	 * are not counted. When first called, 'current' needs to be prev-sibling
	 * of first UI node. 
	 * @param current: staring root of the subtree
	 * @param wanted:  Number of nodes to skip 	
	 * @return; Actual number of nodes skipped.
	 */
	private int skipNodeBackward(UniquePath current, int wanted) {
		int skippedCounter = 0;
		if (!isIncluded(current)) 
			return skippedCounter;
		
		// contrary to forward, root(current)is included at last 
		if (isExpanded(current)!=-1) { // if expanded, go for children
			Node focused = current.getElement(), child = null;
			int length = focused.getNumberOfChildren();
			for (int i = length - 1; i >= 0; i--) {

				UniquePath childPath =
					current.createChildPath(focused.getEdge(i));
				if (!isIncluded(childPath))
					continue;
					
				if (isExpanded(childPath)==-1) {
					skippedCounter++;
					pathBackward = childPath;
					if (skippedCounter >= wanted) {
						return skippedCounter;
					}
				} else { // recursive
					int childSkip =
						skipNodeBackward(childPath, wanted - skippedCounter);
					skippedCounter += childSkip;
					if (skippedCounter >= wanted)
						return skippedCounter;
				}
			} // will loop if node is not enough
		}
		skippedCounter++; // current node
		pathBackward = current;
		return skippedCounter;
	}

	/**
	 * Traverse and retrieve siblings backwards.
	 * @param current: current path as a pointer
	 * @param wanted: number of nodes to be skipped
	 * @return: actual number of nodes skipped
	 */
	private int skipSiblingsBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getPrevSibling();
		while(sibling!=null) {
			int sibSkip=skipNodeBackward(sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getPrevSibling();			
		}
		return skippedCounter;
	}

	/** 
	 * Loops through all parents if there is not enough nodes.
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int skipParentsBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath parent=current.createParentPath(); // one by one
		while (parent!=null) {
			skippedCounter++; // this parent is counted by 1, then its siblings
			pathBackward=parent;
			if (skippedCounter>=wanted) 
				return skippedCounter;			
			skippedCounter+=skipSiblingsBackward(parent,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			parent=parent.createParentPath();
		} // if not enough, recursively search parents			
		return skippedCounter;
	}

	/**
	 * Main skip method to skip given number of nodes.
	 * Delegates calls to skipNode, skipSibling and skipParent.
	 * After skipping, the next path for loading/linking is saved in pathForward.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number skipped
	 */
	private int skipBackward(UniquePath current, int wanted) {
		int skippedCounter=0;
		int toSkip=wanted;
		skippedCounter+=skipSiblingsBackward(current,toSkip-skippedCounter);
		if (skippedCounter<toSkip)
			// if still not, skip all the parents
		skippedCounter+=skipParentsBackward(current,toSkip-skippedCounter);
		return skippedCounter;
	}
	
	/**
	 * Tests if there are enough nodes can be skipped.
	 * @param current: starting path
	 * @param wanted : desired number
	 * @return: actual number that can be skipped
	 */
	private int testSkipBackward(UniquePath current, int wanted) {
		UniquePath old=pathBackward;
		int skipped=skipBackward(current, wanted);
		pathBackward=old;
		return skipped;
	}
	/**
	 * Tests if top of tree has beem reached.
	 * @return: true for top
	 */
//	private boolean testTop() {
//		UniquePath old=pathBackward;
//		int skipped=skipBackward(pathBackward, 1);
//		pathBackward=old;
//		return (skipped==0)?true:false;
//	}

	/**
	 * Link the a node/subtree. Relations between parent/current will be created.
	 * If current is expanded before, it will be treated as a subtree, and all 
	 * the children will be created and linked recursively.
	 * Note: Before calling this method, parent/child SHOULD be not linked. Otherwise
	 * it will be re-linked which case is error. 
	 * @param parent: parent node of current.
	 * @param current: current node to be processed
	 * @param wanted: maximum number of nodes desiried to be linked from current and its children 
	 * @return: actual number linked.
	 */
	private int linkNodeForward(UniquePath parent,UniquePath current, int wanted) {
		int skippedCounter=0;
		if (isIncluded(current)) {
			skippedCounter=1;
			pathForward=current;
			parent.addChild(current);
			current.setParent(parent);
			if ( (skippedCounter>=wanted) || (isExpanded(current)==-1) ) {
				return skippedCounter;
			}
		} else 
			return skippedCounter;
		
		Node focused=current.getElement(),child=null;
		int length=focused.getNumberOfChildren();
		for (int i=0;i<length;i++) {
//			child=focused.getChildAt(i);
//			if (!includeHidden&&child.isHidden())  // hidden child is not counted 
//				continue;
				
			UniquePath childPath=current.createChildPath(focused.getEdge(i));
			if (!isIncluded(childPath))
				continue;			
			childPath.setTrace(trace);
			if (isExpanded(childPath)==-1) {
				current.addChild(childPath);  // link node
				childPath.setParent(current);
				skippedCounter++;
				pathForward=childPath;
				if (skippedCounter>=wanted) {
					return skippedCounter;				
				}
			} else { // recursive
				int childSkip=linkNodeForward(current,childPath,wanted-skippedCounter); // link is done in call
				skippedCounter+=childSkip;
				if (skippedCounter>=wanted) 
					return skippedCounter;
			}				
		} // loop for
	return skippedCounter;
	}
	
	/**
	 * Link siblings of current node
	 * @param parent: parent path to be set in the parentUI field of current path
	 * @param current: current Path 
	 * @param wanted: number of nodes to be retrieved
	 * @return: actual number found
	 */
	private int linkSiblingsForward(UniquePath parent,UniquePath current, int wanted) {
		int skippedCounter=0;
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getNextSibling();
		while(sibling!=null) {
			int sibSkip=linkNodeForward(parent,sibling,wanted-skippedCounter);
			skippedCounter+=sibSkip;
			if (skippedCounter>=wanted) 
				return skippedCounter;
			sibling=(UniquePath)current.getNextSibling();
		}
		return skippedCounter;
	}

	/** 
	 * Loops through all parents if there is not enough nodes.
	 * There is NO parent parameter as in previous node/siblng linking method, 
	 * because parent field can always be found thru getParent(). 
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private int linkParentsForward(UniquePath current, int wanted) {
		int skippedCounter=0;
		UniquePath upper1Path=(UniquePath)current.getParent();
		UniquePath upperParent=(UniquePath)upper1Path.getParent();			
		while ((upper1Path!=null)&&(upperParent!=null)) { // when reach inv, link will terminate
			skippedCounter+=linkSiblingsForward(upperParent,upper1Path,wanted-skippedCounter);
			if (skippedCounter>=wanted) 
				return skippedCounter;
			upper1Path=(UniquePath)upper1Path.getParent();
			upperParent=(UniquePath)upper1Path.getParent();
		} // if not enough, recursively search parents
		return skippedCounter;
	}

	private boolean linkNodeForward(UniquePath parent,UniquePath current, UniquePath tillInclusive) {
		if (!isIncluded(current))
			return false;
			
		parent.addChild(current);
		current.setParent(parent);
		if ( current.equals(tillInclusive) )
			return true;
		else if (isExpanded(current)==-1)
			return false;
		
		Node focused=current.getElement(),child=null;
		int length=focused.getNumberOfChildren();
		for (int i=0;i<length;i++) {
//			child=focused.getChildAt(i);
//			if (!includeHidden&&child.isHidden())  // hidden child is not counted 
//				continue;
				
			UniquePath childPath=current.createChildPath(focused.getEdge(i));
			if (!isIncluded(childPath))
				continue;
			
			childPath.setTrace(trace);
			if (isExpanded(childPath)==-1) {
				current.addChild(childPath);  // link node
				childPath.setParent(current);
				if (childPath.equals(tillInclusive))
					return true;
			} else { // recursive
				boolean found=linkNodeForward(current,childPath,tillInclusive); // link is done in call
				if (found)
					return true;
			}				
		} // loop for
	return false;
	}
	
	/**
	 * Link siblings of current node
	 * @param parent: parent path to be set in the parentUI field of current path
	 * @param current: current Path 
	 * @param wanted: number of nodes to be retrieved
	 * @return: actual number found
	 */
	private boolean linkSiblingsForward(UniquePath parent,UniquePath current, UniquePath tillInclusive) {
		current.resetSibling();
		UniquePath sibling=(UniquePath)current.getNextSibling();
		while(sibling!=null) {
			boolean found=linkNodeForward(parent,sibling,tillInclusive);
			if (found)
				return true;
			sibling=(UniquePath)current.getNextSibling();
		}
		return false;
	}

	/** 
	 * Loops through all parents if there is not enough nodes.
	 * There is NO parent parameter as in previous node/siblng linking method, 
	 * because parent field can always be found thru getParent(). 
	 * @param current: current path
	 * @param wanted: number of path to be retreived
	 * @return: actual number found 
	 */
	private boolean linkParentsForward(UniquePath current, UniquePath tillInclusive) {
		UniquePath upper1Path=(UniquePath)current.getParent();
		UniquePath upperParent=(UniquePath)upper1Path.getParent();			
		while ((upper1Path!=null)&&(upperParent!=null)) { // when reach inv, link will terminate
			boolean found =linkSiblingsForward(upperParent,upper1Path,tillInclusive);
			if (found) 
				return true;
			upper1Path=(UniquePath)upper1Path.getParent();
			upperParent=(UniquePath)upper1Path.getParent();
		} // if not enough, recursively search parents
		return false;
	}
	

	/** Link is always done in forward, no backward
	 * Main Method for link, called other linking methods.
	 * @param parent
	 * @param child
	 * @param wanted
	 * @return
	 */
	private int linkForward(UniquePath parent,UniquePath child, int wanted) {
		int desiredCounter=0;
		desiredCounter+=linkNodeForward(parent,child,wanted-desiredCounter);
		if (desiredCounter>=wanted)
			return desiredCounter;
		desiredCounter+=linkSiblingsForward(parent,child,wanted-desiredCounter);
		if (desiredCounter>=wanted) 
			return desiredCounter;
		desiredCounter+=linkParentsForward(child,wanted-desiredCounter);
		return desiredCounter;				
	}

	/**
	 * Same as the other linkForward method, but link nodes till the given one(as the stop condition).
	 * @param parent
	 * @param child
	 * @param tillInclusive
	 * @return
	 */
	private boolean linkForward(UniquePath parent,UniquePath child, UniquePath tillInclusive) {
		boolean found=linkNodeForward(parent,child,tillInclusive);
		if (found)
			return true;
		found=linkSiblingsForward(parent,child,tillInclusive);
		if (found)
			return true;
		found=linkParentsForward(child,tillInclusive);
		if (found)
			return true;
		else 
			return false;
	}

	/**
	 * Main loading method, called by public interface method.
	 * Two steps first skip number of nodes(forth and back if no enough number),
	 * then load desired number of nodes. 
	 * @param current: starting pointing to for loading
	 * @param wanted:  number of nodes to be loaded into the partial tree
	 * @param skipped: number of nodes to be skipped-it's the offset starts from 0
	 * @param forceRefresh: if true, be sure to relaod tree, otherwise, may keep current for efficiency.
	 * @return:        root of the partial tree after loading
	 */				
	// Clarify ideas: we are creating a partial tree, and reference to the old one.
	// Thereofore, most UniquePaths belongs to new tree. siblings need to be reset when used 
	// determine top4NextRound display is important 
	private UniquePath loadingForward(UniquePath current, int wanted,int skipped,boolean forceRefresh) {
//		if (testBottom()&&!forceRefresh) // reach bottom
//			return invisibleRoot;

		// step1. try to see if all nodes in path are visible
//		Node[] nodes=current.getAllNodes();
//		if (!includeHidden&&nodes[0].isHidden()) { // root is hidden
//			starting.reset(); //clear children
//			return invisibleRoot;
//		}
//		
//		UniquePath lastVisible;
//		if (includeHidden)
//			lastVisible=current;
//		else { 
//			int length=0; // length of edges that is not hidden
//			for (int i=0;i<nodes.length;i++) {
//				if (!nodes[i].isHidden()) 
//					length++;
//				else
//					break;
//			}		
//			lastVisible=new UniquePath(current,length); // check if all node are visble
//		}


		UniquePath lastVisible=getVisiblePath(current);
		if (lastVisible==null) {
			starting.reset(); //clear children
			return invisibleRoot;
		}
			
		// Step 2. only skip where all nodes along the path are visible
		UniquePath realSkipPoint;
		int realWant,realSkip;
		if (lastVisible.equals(current)) { // only skip when no hidden nodes			
			int total=wanted+skipped;
			int test=testSkipForward(lastVisible,total);
			if (test>=wanted) { // enough for last page
				realSkipPoint=lastVisible;
				realWant=wanted;
				realSkip=test-wanted;		
			} else {
				int backNeeded=wanted-test;
				int back=skipBackward(lastVisible,backNeeded); // real move
				realSkipPoint=pathBackward;
				realWant=test+back;
				realSkip=0;
			}					
		} else { // no skip
			int total=wanted;
			int test=testSkipForward(lastVisible,total);
			if (test>=wanted) { // enough for last page
				realSkipPoint=lastVisible;
				realWant=wanted;
				realSkip=0;		
			} else {
				int backNeeded=wanted-test;
				int back=skipBackward(lastVisible,backNeeded); // real move
				realSkipPoint=pathBackward;
				realWant=test+back;
				realSkip=0;
			}								
		}
		
		lastSkippedCounter=skipForward(realSkipPoint,realSkip);
		if (realSkip>0) { // from new point
			top4NextRound=pathForward;
		} else { // use realLoadPoint since no skip
			top4NextRound=realSkipPoint;
		}
		pathBackward=top4NextRound; // adjust pointer for backward

		starting.reset();
		Edge[] ns1=top4NextRound.getAllEdges();
		UniquePath parent=starting,child=null;
		for (int i=0;i<ns1.length;i++) {
			child=parent.createChildPath(ns1[i]);
			if (i!=ns1.length-1) {// last one is not lined here, link is done universally in linkNodeForward
				parent.addChild(child);  // link all other nodes except last one
				child.setParent(parent);
				parent=child;
			}
		}
		if (child==null) {// if pathForward is main, will skip above for
			parent=invisibleRoot;
			child=starting;
			parent.reset();
			child.reset();
		}
		
//		if ((realWant+1)<(wanted+skipped))
//			linkForward(parent,child,realWant+1); // plus one for current
//		else
//			linkForward(parent,child,realWant); // plus one for current
		linkForward(parent,child,realWant); // plus one for current
		return invisibleRoot;
	}

	private UniquePath loadingForward(UniquePath from, UniquePath till) {
		UniquePath starting2=new UniquePath(starting);
		starting2.setDescription("starting");
		starting2.setTrace(trace);

		UniquePath invisibleRoot2=new UniquePath(invisibleRoot); // root of UI tree to be returned, it's the same path as starting
		invisibleRoot2.setDescription("Invisible");		
		invisibleRoot2.setTrace(trace);
		
		// step1. try to see if all nodes in path are visible
//		Node[] nodes=from.getAllNodes();
//		if (!includeHidden&&nodes[0].isHidden()) { // root is hidden
//			return invisibleRoot2;
//		}
//
//		invisibleRoot2.addChild(starting2);
//		starting2.setParent(invisibleRoot2);		
//		UniquePath lastVisible;
//		if (includeHidden)
//			lastVisible=from;
//		else { 
//			int length=0; // length of edges that is not hidden
//			for (int i=0;i<nodes.length;i++) {
//				if (!nodes[i].isHidden()) 
//					length++;
//				else
//					break;
//			}		
//			lastVisible=new UniquePath(from,length); // check if all node are visble
//		}
		
		if (!isIncluded(from)||!isIncluded(till))
			return invisibleRoot2;
			
		UniquePath lastVisible=from;
		Edge[] ns1=lastVisible.getAllEdges();
			UniquePath parent=starting2,child=null;
			for (int i=0;i<ns1.length;i++) {
				child=parent.createChildPath(ns1[i]);
				if (i!=ns1.length-1) {// last one is not lined here, link is done universally in linkNodeForward
					parent.addChild(child);  // link all other nodes except last one
					child.setParent(parent);
					parent=child;
				}
			}
			if (child==null) {// if pathForward is main, will skip above for
				parent=invisibleRoot2;
				child=starting2;
				parent.reset();
				child.reset();
			}

		boolean found =linkForward(parent,child,till); // plus one for current
		if (!found)
			invisibleRoot2.reset();
		else if (invisibleRoot2.getChildren().length==0){
			invisibleRoot2.addChild(starting2);
			starting2.setParent(invisibleRoot2);
		}
		return invisibleRoot2;
	}

	/** 
	 * Loads children from a previous point.
	 * @param current
	 * @param wanted
	 * @param skipped
	 * @param forceRefresh
	 * @return
	 */
	private UniquePath loadingBackward(UniquePath current, int wanted,int skipped,boolean forceRefresh) {
//		if (testTop()&&!forceRefresh) // reach bottom
//			return invisibleRoot;

		// step1. try to see if all nodes in path are visible
//		Node[] nodes=current.getAllNodes();
//		if (!includeHidden&&nodes[0].isHidden()) { // root is hidden
//			starting.reset(); //clear children
//			return invisibleRoot;
//		}
//
//		UniquePath lastVisible;
//		if (includeHidden)
//			lastVisible=current;
//		else { 
//			int length=0; // length of edges that is not hidden
//			for (int i=0;i<nodes.length;i++) {
//				if (!nodes[i].isHidden()) 
//					length++;
//				else
//					break;
//			}		
//			lastVisible=new UniquePath(current,length); // check if all node are visble
//		}

		UniquePath lastVisible=getVisiblePath(current);
		if (lastVisible==null) {
			starting.reset(); //clear children
			return invisibleRoot;
		}

		// Step 2. only skip where all nodes along the path are visible
		UniquePath realSkipPoint;
		int realWant,realSkip;
		if (lastVisible.equals(current)) { // only skip when no hidden nodes			
			realSkip=skipBackward(lastVisible,skipped);
			realSkipPoint=pathBackward;
			realWant=testSkipForward(realSkipPoint,wanted);			
		} else { // no skip
			realSkip=0;		
			realSkipPoint=lastVisible;
			realWant=testSkipForward(realSkipPoint,wanted);
		}
		
		top4NextRound=realSkipPoint;
		lastSkippedCounter=realSkip;
		
		starting.reset();
		Edge[] ns1=top4NextRound.getAllEdges();
		UniquePath parent=starting,child=null;
		for (int i=0;i<ns1.length;i++) {
			child=parent.createChildPath(ns1[i]);
			if (i!=ns1.length-1) {// last one is not lined here, link is done universally in linkNodeForward
				parent.addChild(child);  // link all other nodes except last one
				child.setParent(parent);
				parent=child;
			}
		}
		if (child==null) {// if pathForward is main, will skip above for
			parent=invisibleRoot;
			child=starting;
			parent.reset();
			child.reset();
		}


//		linkForward(parent,child,realWant+1); // plus one for current
		linkForward(parent,child,realWant); // plus one for current
		return invisibleRoot;
	}		
	
	/**
	 * Determines if all the intermediate nodes along the given path are visible.
	 * Return a trunct path before its reaches the invisible nodes on the path.   
	 * @param current
	 * @return: truncted path
	 */	
	private UniquePath getVisiblePath(UniquePath current) {
		// step1. try to see if all nodes in path are visible
		UniquePath tructed;
		for (int i=1;i<=current.getLength();i++) {
			tructed=new UniquePath(current,i);
			if (!isIncluded(tructed))
				return tructed.createParentPath();
		}
		return current;	
	}
	

	
	/**
	 * Loads initial data when the system starts.
	 * @return: root
	 */			
	public UniquePath loadingStartup() {
		// clear relation first, they will be setup universally by link
		invisibleRoot.reset();
		starting.reset();
		top4NextRound=pathForward=pathBackward=starting;
		addToExpandSet(starting);
		loadingForward(starting, this.sizeNeeded,0, true);
		return invisibleRoot;		
	}

	/**
	 * Loads initial data when the system starts.
	 * @return: root
	 */			
	public UniquePath loadingStartup(int treeSize) {
		// clear relation first, they will be setup universally by link
		setTreeSize(treeSize);
		return loadingStartup();
//		invisibleRoot.reset();
//		starting.reset();
//		top4NextRound=pathForward=pathBackward=starting;
//		loadingForward(starting, this.sizeNeeded,0, true);
//		return invisibleRoot;		
	}	

	/**
	 * Adds the path to the expanded set. This method is normally called when a path
	 * is first added to the expanded node list. The state is determined by
	 * isAllRawChildrenShown() for the given path.
	 * @param path
	 */
	public void addToExpandSet(UniquePath path) {
		// use HashSet for list
		if (path.getElement().isLeaf()) // doing for a leaf node
			return;
		int state=isAllRawChildrenShown(path)?PictureLabel.MINUS:PictureLabel.WAVE;
		addToExpandSet(path,state);
//		expandedSet.put(path,new Integer(state));
	}

	/**
	 * Add the path to the expanded path set. The state is either partially expanded(WAVE) or
	 * fully expanded(MINUS). 
	 * @param path
	 * @param state
	 */
	public void addToExpandSet(UniquePath path, int state) {
		// use HashSet for list
		removeFromExpandSet(path);
		expandedSet.put(path,new Integer(state));
	}
	
	/**
	 * Removes the path from the expanded set
	 * @param path
	 */
	public void removeFromExpandSet(UniquePath path) {
		UniquePath found=null;
		Enumeration en=expandedSet.keys();
		while (en.hasMoreElements()) {
			UniquePath next=(UniquePath)en.nextElement();
			if (next.equals(path)) {
				found=next;
				break;
			}
		}
		if (found!=null)
			expandedSet.remove(found);
	}
	
	/**
	 * Refreshes all the expand/collpase states to reflects data change. 
	 * @param force-true: ignore user clicks which make changes between +/~
	 * 			    false: keep user clicks settings
	 */
	public void refreshAllExpansion(boolean force) {
		Enumeration en=expandedSet.keys();
		while (en.hasMoreElements()) {
			UniquePath next=(UniquePath)en.nextElement();	
			if (force)
				removeFromExpandSet(next);		
			addToExpandSet(next);
		}
	}
	
	/**
	 * Tests if the path is opened.
	 * @param path
	 * @return: integer: MINUS, WAVE, or NONE
	 */
	public int isExpanded(UniquePath path) {
		Enumeration en=expandedSet.keys();
		while (en.hasMoreElements()) {
			UniquePath next=(UniquePath)en.nextElement();
			if (next.equals(path)) {
				return ((Integer)expandedSet.get(next)).intValue();
			}
		}		
		return -1;
	}		
	
	/**
	 * Tests if the all parent pathes of this path is opened.
	 * @param path
	 * @return: boolean
	 */
	public boolean isParentsExpanded(UniquePath path) {
		for (int i=1;i<=path.getLength();i++) {
			UniquePath par=new UniquePath(path,i);
			if (isExpanded(par)==-1)
					return false;
		}
		return true;
	}	

	/**
	 * Tests if the all parent pathes of this path is includes.
	 * @param path
	 * @return: boolean
	 */
	public boolean isParentsIncluded(UniquePath path) {
		for (int i=1;i<=path.getLength();i++) {
			UniquePath par=new UniquePath(path,i);
			if (!isIncluded(par))
					return false;
		}
		return true;
	}	

			
	/**
	 * Determines whether a path is to be included in the retrieval.
	 * Precondition of this method: The direct parent of given path 
	 * is included already. 
	 * Several conditions apply here:
	 * 1. Global parameter if hidden nodes are included;
	 * 2. Parent local +/~ sign that override the global setting
	 * 3. The internal state of path itself(show/hide) 
	 * If the parent has not expanded yet, the second condition will be ignored.
	 * @param path
	 * @return
	 */
	public boolean isIncluded(UniquePath path) {
		boolean internalResult;
		int expState=isExpanded(path.createParentPath());
		if (includeHidden) {
			if (expState==PictureLabel.WAVE)  // for a wave sign, only those path are NOT hidden will be included
				internalResult=(path.getElement().isHidden())?false:true;
			else
				internalResult=true;
		} else {  
			if (path.getElement().isHidden()) 
				internalResult=(expState==PictureLabel.MINUS)?true:false;
			else 
				internalResult=true;
		}
		return internalResult&&filter;
	}

	public boolean isIncluded(UniquePath parentPath,int childIndex) {
		boolean internalResult;
		Node child=parentPath.getElement().getChildAt(childIndex);
		int expState=isExpanded(parentPath);
		
		if (includeHidden) {
			if (expState==PictureLabel.WAVE)  // for a wave sign, only those path are NOT hidden will be included
				internalResult=(child.isHidden())?false:true;
			else
				internalResult=true;
		} else {  
			if (child.isHidden()) 
				internalResult=(expState==PictureLabel.MINUS)?true:false;
			else 
				internalResult=true;
		}
		return internalResult&&filter;
	}
		
	/**
	 * Returns if all children path from model will be shown or not.
	 * The method is used to determine the +/~ sign for a parent path.
	 * @param path
	 * @return
	 */
	public boolean isAllRawChildrenShown(UniquePath path) {
		Node node =path.getElement();
		int length=node.getNumberOfChildren();
		for (int i=0;i<length;i++) {
			if (!isIncluded(path,i))
				return false;
		}
		return true;	
//		boolean internalResult;
//		if (includeHidden)  // this if is only for normal "include"
//			internalResult=true;
//		else {
//			if (path.hasHiddenRawChildren())
//				internalResult=false;
//			else 
//				internalResult=true;
//		}
//		return internalResult&&filter;
	} 
	
	/**
	 * Tests if there are more siblings that can be included in the creation
	 * of tree. This method if used in rendering(drawing) of the tree so
	 * that there is a indication that more pathes are there.
	 * @param path
	 * @return
	 */
	public boolean hasMoreSilbingToShow(UniquePath path) {
		path.resetSibling();
		UniquePath sibling=(UniquePath)path.getNextSibling();
		while(sibling!=null) {
			if (isIncluded(sibling))
				return true;
			sibling=(UniquePath)path.getNextSibling();
		}
		return false;
	}	
	public void setFilter(boolean condition) {
		filter=condition;
	}
	
	public boolean getFilter() {
		return filter;
	}

	
	public UniquePath getTop4NextRound() {
		return top4NextRound;
	}

	/** Returns the number of nodes that the expanded tree should actually contains.
	 *  But the partial tree have less that this number
	 *  @return: tree size
	 */
	public int getWholeTreeSize() {
		Enumeration en=expandedSet.keys();
		int counter=1;// +1 one for main	
		while (en.hasMoreElements()) {
			UniquePath expandedPath=(UniquePath)en.nextElement();
			if (!isParentsExpanded(expandedPath)||!isParentsIncluded(expandedPath)||!isIncluded(expandedPath)) // do not count
				continue;
			Node expandedNode=expandedPath.getElement();
			int length=expandedNode.getNumberOfChildren();
			for (int i=0;i<length;i++) {
				if (isIncluded(expandedPath,i))
					counter++;
			}
		}
		return counter; 
	}

	/**
	 * Set first visible node to be retrieved for the partial tree.
	 * This method also sets nodes along this pathes to be expnaded
	 * if that path is not done before. 
	 * @return
	 */	
	public void setLoadingPoint(UniquePath startPoint) {
		for (int i=1;i<startPoint.getLength();i++) {// top is not included
			UniquePath intermediate=new UniquePath(startPoint,i); 
			addToExpandSet(intermediate);
		}		
	}

	/**
	 * Gets the offset of current top node from the root.
	 * This offset is used to indicate to the user the current position,
	 * and to set proper vertical scroll position. It iterates thru the expanded
	 * nodes list and use comparison to determine which node is before the current
	 * and which is after or equals
	 * 
	 * @return int: between 0-getTreeCount-1. or -1 if an error occurs
	 */
	public int getOffsetFromRoot(UniquePath startPoint) {
		if (startPoint.equals(starting)) // test for main root node
			return 0;
		try {
			Enumeration en=expandedSet.keys();
			int offset=0;
			while (en.hasMoreElements()) {
				UniquePath expandedPath=(UniquePath)en.nextElement();
				if (!isIncluded(expandedPath)) // tests if it is calculated
					continue;
				if (expandedPath.isAncestorPath(startPoint)) {
					// only count part of pathes that are siblings before the stop psth
					UniquePath stop=new UniquePath(startPoint,expandedPath.getLength()+1); 
					Node expandedNode=expandedPath.getElement();
					int length=expandedNode.getNumberOfChildren();
					for (int i=0;i<length;i++) {
						UniquePath child=expandedPath.createChildPath(expandedNode.getEdge(i));
						if (compareOrder(child,stop)!=1) { // count for before or equal
						   if (isIncluded(child))
								offset++;
						} else
							break;
					}								
				} else if (compareOrder(expandedPath,startPoint)==-1) {
					// if a path in the expand set is before this one, count all its directed children.
					// Note: indirect children will be calculated in other expanded nodes, as iteration goes. 
					Node expandedNode=expandedPath.getElement();
					int length=expandedNode.getNumberOfChildren();
					for (int i=0;i<length;i++) {
						UniquePath child=expandedPath.createChildPath(expandedNode.getEdge(i));
						if (isIncluded(child))
							offset++;
					}				
				}
			}
			//System.out.println("Offset:"+offset);
			return offset;
		} catch (Exception e) {
			e.printStackTrace();
			return -1;
		} 
	}
	
	/**
	 * Compares which path comes first. Assumption: These paths start from
	 * same root node, main. Otherwise, comparison can not be determined,  
	 * and an expception is throwed
	 * @param path1
	 * @param path2
	 * @return -1: path1 is less than (before) path2 , 0 equal, 1: path1 is greater than (after) path2
	 */
	private int compareOrder(UniquePath path1, UniquePath path2) throws Exception{
		Edge[] edges1=path1.getAllEdges();  // note: not include dumb edge
		Edge[] edges2=path2.getAllEdges();
		if (edges1.length<1)
		    return -1;
		if (edges2.length<1)
			return 1;
		Node root1=edges1[0].getFirstNode();
		Node root2=edges2[0].getFirstNode();
		if (root1!=root2) 
			throw new Exception("Path Comparison: starting from different roots can not continue!");
		Node parent=root1;
		int shorter=(edges1.length<edges2.length)?edges1.length:edges2.length;
		for (int i=0;i<shorter;i++) {
			if (edges1[i]!=edges2[i]) { // compare two siblings
				for (int k=0;k<parent.getNumberOfChildren();k++) {
					Edge ce=parent.getEdge(k);
					if (ce==edges1[i])   
						return -1;
					else if (ce==edges2[i])
						return 1;					
				}
				throw new Exception("Path Comparison: Cannot determine from parent!");									
			} else
				parent=edges1[i].getSecondNode();
		}
		
		// till this far, shorter parts are same
		return (edges1.length<edges2.length)?-1:(edges1.length>edges2.length)?1:0;
	}
//	/**
//	 * Returns offset of current path, so it can be sued to set Scrollbar position  
//	 */
//	public int calcFocusOffset() {
//		return this.totalVisibleOffest;
//	}
	

	// helper method to display the tree found
	public void printing(UniquePath path)
	{
		temp=1;
	   restoreRepHelper(path,0);
	   Enumeration en=expandedSet.keys();
	   System.out.println("--------Expandedlist-----");	   	
	   while (en.hasMoreElements()) {
			UniquePath up=(UniquePath)en.nextElement();
			System.out.println(" " +up.getName());	   	
	   }
	   System.out.println("--------end of list-----");	   	
	}
  
	// called by printing
	int temp=1;
	private void restoreRepHelper(UniquePath path,int level)
	{
		
		if (path==null)
			return;
		System.out.print(""+temp);
		temp++;
		for (int i=0;i<level;i++)
			System.out.print("  ");
	 	System.out.println(path.getName());
	 	TreeNode[] children=(TreeNode[])path.getChildren();
	 	for (int i=0;i<children.length;i++) {
			restoreRepHelper((UniquePath)children[i],level+1);
	 	}
  }

	/**
	 * Constructs an initial partial tree with size 10.
	 */
	public TreeNode construct() {
		// TODO Auto-generated method stub
		setTreeSize(10);
		loadingStartup();
		return getRoot();
	}

	/**
	 * Returns the root node of the tree.
	 */
	public UniquePath getRoot() {
		// TODO Auto-generated method stub
		return starting;
	}
	/**
	 * Returns the invisible nodes of tree, which is the parent of ROOT 
	 * and is used to feed it to the UI tree.
	 *	 */
	public TreeNode getInvisibleRoot() {
		// TODO Auto-generated method stub
		return invisibleRoot;
	}
	
//	/**
//	 * @return
//	 */
//	public Hashtable getLastColoredModelNode() {
//		return lastColoredModelNode;
//	}
//
//	/**
//	 * @param node
//	 */
//	public void setLastColoredModelNode(Node node,int color) {
//		lastColoredModelNode.clear();  
//		if (node==null)	return;
//		lastColoredModelNode.put(node,new Integer(color));
////		if (node.getNumberOfChildren()<=0)
////			return;
////		Enumeration en=node.getChildren();
////		while (en.hasMoreElements()) {
////			lastColoredModelNode.put(en.nextElement(),new Integer(color));			
////		}
//	}
	

	/**
	 * @return
	 */
	public Trace getTrace() {
		return trace;
	}

}
