package seat.ui.tree.viewer;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelection;

import seat.io.TraceSession;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.ui.algorithm.AlgorithmList;
import seat.ui.algorithm.NodeBits;
import seat.ui.tree.partial.TreeNode;
import seat.ui.views.MethodView;
import seat.ui.views.ModelView;

/**
 * An exploration is an interaction with a trace. Each time a trace is opened,
 * an exploration will be created. Multiple explorations can be conducted on the trace
 * and they reside in the different pages of a multiple trace editor. Two explorations
 * of the same trace do not interfere with each other as if they are different traces.
 * 
 * Switching editor and editor page within an editor will cause a switch of explorations.
 * Most user realted data are kept in exploration. Care should be taken in switching 
 * so synchronization is achieved. 
 */
public class Exploration {
	// data from model
	private Trace trace;								// trace model
	private TraceSession session=new TraceSession();    // delegate some processing to session
	private AlgorithmList algorithms=new AlgorithmList();// algorithms applied in current exploration	

	private Hashtable modelNodes; 						// states of each INTERNAL model nodes:Integer
	private Hashtable modelStates;						// states of each nodes if they are left out by user click:Boolean
	
	// input for auxillary views
	private ArrayList modelList=new ArrayList(); 		// utility method list(holds LogicalPackage/ClassElement/Node objects)
	private ArrayList utilityList=new ArrayList(); 		// utility method list(holds Node objects)
	private ArrayList patternList=new ArrayList();		// pattern list(holds Node objects) 
	private Hashtable userContextNodes=new Hashtable(); // holds nodes that user right clicked.

	// previous selection for auxillary views
	private ModelView modelView;
	private MethodView utilityView,patternView;
	private ISelection modelSelection,utilitySelection,patternSelection;
	
	// flags for adding package and class list
	boolean pckAdded=false,clsAdded=false;
	public Exploration(Trace trace) {
		this.trace=trace;
		addParsedPackageClass();
		modelList.addAll(trace.getState().getModelNodeList());
		modelNodes=new Hashtable(); // those are not Node, set int to 0
		modelStates=new Hashtable(); // those are not Node, set int to 0
		Hashtable initial=trace.getState().getInitialNodeStates();
		for (int i=0;i<modelList.size();i++) {
			Object obj=modelList.get(i);
			if (initial.containsKey(obj))
				modelNodes.put(obj,initial.get(obj));
			modelStates.put(obj,new Boolean(false)); // not checked
		}
	}

	// test if packages and classes are already added
	private void addParsedPackageClass() {
		if (!clsAdded) {
		   int oldSize=modelList.size();
		   modelList.addAll(0,trace.getState().getClassList());
		   if (modelList.size()!=oldSize) {
//		   		for (int i=0;i<trace.getState().getClassList().size();i++) {
//					if (trace.getState().getClassList().get(i)==null)
//		   				System.out.println("classl ist element====null");
//		   			else
//					modelStates.put(trace.getState().getClassList().get(i),new Boolean(false));
//		   		}
			clsAdded=true;		 	
		   }
		}
		if (!pckAdded) {
		   int oldSize=modelList.size();
		   modelList.addAll(0,trace.getState().getPackageList());
		   if (modelList.size()!=oldSize) {
//				for (int i=0;i<trace.getState().getPackageList().size();i++)
//					modelStates.put(trace.getState().getPackageList().get(i),new Boolean(false));
			pckAdded=true;		 	
		   }
		}
	}

	/**
	 * Synchronizes exploration from trace data. Exploration will be modified according to 
	 * the data in trace. This method will usually be called when an de-activation 
	 * of an editor/editor page.
	 */
	public void syncExplorationFromTrace() {
		addParsedPackageClass();
		// save model nodes in exploration
		for (int i=0;i<modelList.size();i++) {
			Object obj=modelList.get(i);
			if (obj instanceof Node) {
				Integer bit=new Integer(((Node)obj).getBits());
				modelNodes.put(obj,bit);
			}
		}
		// save check state for each item in ModelView
		if (modelView!=null) {
			CheckboxTableViewer ctv=(CheckboxTableViewer)modelView.getViewer();
			for (int i=0;i<modelList.size();i++) {
				Object obj=modelList.get(i);
				Boolean check=new Boolean(ctv.getChecked(obj));
				modelStates.put(obj,check);
			}
			// save ModeView selection
			modelSelection=modelView.getViewer().getSelection();			
		}
		// save UtilityView selection
		if (utilityView!=null) 
			utilitySelection=utilityView.getViewer().getSelection();
		// save Patternview selection
		if (patternView!=null) 
			patternSelection=patternView.getViewer().getSelection();
	}
	
	/**
	 * Synchronizes trace data from exploration. Trace internal data will be modified 
	 * with data in Exploration. This method will usually be called when an activation 
	 * of new editor/editor page. The auxillary views will be refreshed to reflect new data.
	 *
	 */
	public void syncTraceFromExploration() {
		addParsedPackageClass();
		// set model nodes
		for (int i=0;i<modelList.size();i++) {
			Object obj=modelList.get(i);
			if (obj instanceof Node) {
				Integer bit=(Integer)modelNodes.get(obj);
				((Node)obj).setBits(bit.intValue());
			}
		}
		// set check state for each item in ModelView
		if (modelView!=null) {
			CheckboxTableViewer ctv=(CheckboxTableViewer)modelView.getViewer();
			ArrayList filtered=modelView.getFilteredElements();
			filtered.clear();
			for (int i=0;i<modelList.size();i++) {
				Object obj=modelList.get(i);
				Boolean check=(Boolean)modelStates.get(obj);
//				if (check.booleanValue())
//					modelView.showUserSelectedNodes(obj);
//				else
//					modelView.hideUserSelectedNodes(obj);
				boolean status=(check==null)?false:check.booleanValue();
				ctv.setChecked(obj,status);
				if (status)  // added to filtered list
					filtered.add(obj);
			}
			// set ModeView selection
			modelView.refresh(modelList);
			modelView.getViewer().setSelection(modelSelection);						
		}
		// set UtilityView selection
		if (utilityView!=null) {
			utilityView.refresh(utilityList);
			utilityView.getViewer().setSelection(utilitySelection);
		}
		// set Patternview selection
		if (patternView!=null) { 
			patternView.refresh(patternList);
			patternView.getViewer().setSelection(patternSelection);
		}
	}
	

	/**
	 * Saves the node and algorithms that applies on it 
	 * @param tn: a node
	 * @param operationIndex: algorithm unique index
	 * @param applyIt: whether it's an apply or undo
	 */
	public void cacheNodeContext(TreeNode tn,int operationIndex,boolean applyIt) {
		NodeBits nb=null;
		TreeNode existing=findInCache(tn);
		if (existing!=null)
			nb=(NodeBits)userContextNodes.get(existing);
		if (nb==null)
			nb=new NodeBits(operationIndex);
		if (applyIt) 
			nb.apply(operationIndex);
		else
			nb.revoke(operationIndex);			
		userContextNodes.put(existing,nb);
	}	
	
	/**
	 * Gets the cached context of a given tree node.
	 * The context is bits of algorithms that are applied on this node.
	 * @param tn- given path
	 * @return -NodeBits
	 */
	public NodeBits getCachedNodeContext(TreeNode tn) {
		NodeBits nb=null;
		TreeNode existing=findInCache(tn);
		if (existing!=null)
			nb=(NodeBits)userContextNodes.get(existing);
		if (nb==null) {
			nb=new NodeBits();
			userContextNodes.put(tn,nb);		
		}
		return nb;		
	}
	
	private TreeNode findInCache(TreeNode tn) {
		TreeNode found=null;
		Enumeration en=userContextNodes.keys();
		while (en.hasMoreElements()) {
			TreeNode next=(TreeNode)en.nextElement();
			if (next.equals((tn))) {
				found=next;
				break;
			}
		}
		return found;
	}
	
	/**
	 * Returns the AlgorithmList object for toolbar.
	 * @return
	 */
	public AlgorithmList getAlgorithmList() {
		return algorithms;
	}
	
	/**
	 * Pattern list of current exploration.
	 * @return
	 */
	public ArrayList getModelList() {
		addParsedPackageClass();
		return modelList;
	}
	
	/**
	* Pattern list of current exploration.
	* @return
	*/
   public ArrayList getPatternList() {
	   return patternList;
   }

	/**
	 * Utility list of current exploration.
	 * @return
	 */
	public ArrayList getUtilityList() {
		return utilityList;
	}
	
//	/**
//	 * Cache of user right clicked nodes
//	 * @return
//	 */
//	public Hashtable getContextNodesCache() {
//		return userContextNodes;
//	}
	
	/**
	 * Session object associated with exploration.
	 * @return
	 */
	public TraceSession getSession() {
		return session;
	}

	/**
	 * Trace object associated with exploration.
	 * @return
	 */
	public Trace getTrace() {
		return trace;
	}

	/**
	 * @param view
	 */
	public void setModelView(ModelView view) {
		modelView = view;
	}

	/**
	 * @param view
	 */
	public void setPatternView(MethodView view) {
		patternView = view;
	}

	/**
	 * @param view
	 */
	public void setUtilityView(MethodView view) {
		utilityView = view;
	}

}
