/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package seat.ui.views;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.search.ui.IActionGroupFactory;
import org.eclipse.search.ui.IGroupByKeyComputer;
import org.eclipse.search.ui.ISearchResultView;
import org.eclipse.search.ui.ISearchResultViewEntry;
import org.eclipse.search.ui.SearchUI;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.ui.algorithm.NameParser;
import seat.ui.tree.partial.PartialTree;
import seat.ui.tree.partial.UniquePath;
import seat.ui.tree.viewer.MultiTraceEditor;
import seat.ui.tree.viewer.TraceEditorPage;

/**
 * Help Search Operation.
 */
public class TraceSearchOperation extends WorkspaceModifyOperation {
	private static String TRACE_SEARCH_PAGE_ID="seat.ui.views.TraceSearchPage";
	
	// Note for debug: Marker Name Issue
	// Marker id definition in plugin xml is a simple name inside platform, it will be
	// prefixed with extra plugin name. Therefore an actual marker id should be:
	// plugin name + marker id, as the following string TRACE_MARKER  	
	public static String TRACE_MARKER="seat.tracenode";
	public static String EXPLORATION_ID="exploration.id";
	public static String LABEL_ATTRIB="node.label";
	public static String ID_ATTRIB="node.id";
	public static String TYPE_ATTRIB="node.type";
	public static String LINE_NO="node.lineno";
	
	// Images
	private static final ImageDescriptor imgSearch =SeatUtil.getImageDescriptor("icons/search.gif");	
	private static final Image imgSubtree = SeatPlugin.getImage(SeatUtil.ICON_COMPOUND);
	private static final Image imgMethod = SeatPlugin.getImage(SeatUtil.ICON_METHOD);
	private static final IResource resource =ResourcesPlugin.getWorkspace().getRoot();

	IProgressMonitor fMonitor;
	private String queryData = null;
	private boolean methodMatch;
	
	private List searchResult;
	private TraceEditorPage page;
	private Trace trace;
	/**
	 * TraceSearchOperation constructor.
	 * @param data SearchQueryData
	 */
	public TraceSearchOperation(String searchString,boolean exact, List searchResult) {
		queryData = searchString;
		this.methodMatch=exact;
		this.searchResult=searchResult;
		IEditorPart active=SeatUtil.getActiveEditor();
		if (active instanceof MultiTraceEditor) {
			trace=((MultiTraceEditor)active).getTrace();
			page=((MultiTraceEditor)active).getCurrentEditor();
		}
	}
	/**
	 * @see WorkspaceModifyOperation#execute(IProgressMonitor)
	 */
	protected void execute(IProgressMonitor monitor)
		throws CoreException, InvocationTargetException, InterruptedException {
		IProgressMonitor fMonitor=monitor;
		monitor.beginTask("Begin Searching",100);
		try {
			doSearch(monitor);
			displayResults();
		} catch (OperationCanceledException oce) {
			monitor.done();
			throw oce;
		} catch (Exception e) {
			e.printStackTrace();
			monitor.done();
		}
		monitor.done();
	}
	
	private void doSearch(IProgressMonitor monitor) {
		if (trace!=null) {
			ArrayList allNodes=trace.getState().getModelNodeList();
			for (int i=0;i<allNodes.size();i++) {
				Node node=(Node)allNodes.get(i);
				if (isQualified(node))
					searchResult.add(node);
			}
		}
	}

	private boolean isQualified(Node node) {
		if (methodMatch)
			return NameParser.getShortMethodName(node).startsWith(queryData);			
		else 
			return (node.getLabel().equals(queryData))? true	: false;
	}
	
	
	
	private void displayResults() {
		ISearchResultView sView = SearchUI.getSearchResultView();
		if (sView != null)
			sView.searchStarted(
					(IActionGroupFactory) null,
					"found 1 match","found {0} matches",
					imgSearch,
					TRACE_SEARCH_PAGE_ID,
					new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof ISearchResultViewEntry) {
					ISearchResultViewEntry entry =(ISearchResultViewEntry) element;
					PathGroup pg=(PathGroup)entry.getGroupByKey();
					return pg.getName();				 
				}
				else
					return "";
			}
			public Image getImage(Object element) {
				if (element instanceof ISearchResultViewEntry) {
						ISearchResultViewEntry entry =(ISearchResultViewEntry) element;
						int type=entry.getSelectedMarker().getAttribute(TYPE_ATTRIB,1);					 
						return (type==1)?imgMethod:imgSubtree;
				} else
					return null;
			}
		}, new org.eclipse.jface.action.Action() {
			// this method will be invoked either a previous or next loval icon is clicked
			public void run() {
				ISearchResultView view = SearchUI.getSearchResultView();
				ISelection selection = view.getSelection();
				Object element = null;
				if (selection instanceof IStructuredSelection)
					element =
						((IStructuredSelection) selection).getFirstElement();
				if (element instanceof ISearchResultViewEntry) {
					ISearchResultViewEntry entry =
						(ISearchResultViewEntry) element;
					try {
						gotoEditor(entry);
					} catch (Exception e) {
						System.out.println(e);
					}
				}
			}
		}, new IGroupByKeyComputer() {
			public Object computeGroupByKey(IMarker marker) {
				try {
					if (marker.getAttribute(LABEL_ATTRIB)!=null)
						return marker.getAttribute(LABEL_ATTRIB);
				} catch (CoreException ce) {
				}
				return "UNKNOWN";
			}
		}, this);
		// Delete all previous results
		try {
			resource.deleteMarkers(
				TRACE_MARKER,
				true,
				IResource.DEPTH_INFINITE);
		} catch (CoreException ex) {
		}
		createResultsMarkers(sView);
		sView.searchFinished();
	}
	
	// Search view framework will automatically calculate number of marker and which marker
	// is selected and jump to next group when current group is end.
	// Therefore, all markers must be created and add to the view and this creates
	// a problem when adding large number of markers, say thousands.
	// description para of addMatch() is not used in version2.1.1 of eclipse  
	private void createResultsMarkers(ISearchResultView sView) {
		if (page==null)
			return;
		PartialTree ptree=(PartialTree)page.getViewer().getTree();
		for (int i = 0; i < searchResult.size(); i++) {
			Node node=(Node)searchResult.get(i);
			try {
				IMarker marker = null;
				String label=node.getLabel();
				int type=node.isLeaf()?1:2;  // 1 for a leaf, 2:subtree
				PathGroup pg=new PathGroup(ptree,node);
				for (int jj=0;jj<pg.getOccurrence();jj++) { 
					// This loop is very slow due to the high occurrrences
					marker = resource.createMarker(TRACE_MARKER); // marker.setAttributes
					marker.setAttribute(LABEL_ATTRIB,label);
					marker.setAttribute(ID_ATTRIB,jj); // use index(to identify which path);
					marker.setAttribute(TYPE_ATTRIB,type);
					sView.addMatch(node.getLabel(),pg,resource,marker);
				}
			} catch (CoreException ce) {
			}
		}
	}
	
	private void gotoEditor(ISearchResultViewEntry entry) {
//		try {
			PathGroup pg=(PathGroup)entry.getGroupByKey();	
			int id=entry.getSelectedMarker().getAttribute(ID_ATTRIB,0);	
			UniquePath next=pg.find(id);
			IEditorPart editor=SeatUtil.getActiveEditor();
			if (editor instanceof MultiTraceEditor) {
				((MultiTraceEditor)editor).getCurrentEditor().getViewer().setHighlightedNode(pg.getNode());
				((MultiTraceEditor)editor).getCurrentEditor().refresh(next);		
			}		
//		} catch (CoreException ce) {}
//	}

		
//		PathGroup pg=(PathGroup)entry.getGroupByKey();	
//		UniquePath next=pg.next();
//		IEditorPart editor=SeatUtil.getActiveEditor();
//		if (editor instanceof MultiTraceEditor) {
//			((MultiTraceEditor)editor).getCurrentEditor().getViewer().setHighlightedNode(pg.getNode());
//			((MultiTraceEditor)editor).getCurrentEditor().refresh(next);		
//		}
	}
}
